<?php

namespace App\Http\Controllers;

use GuzzleHttp\Client;
use App\Models\Country;
use App\Models\PdfSize;
use App\Models\ProductType;
use App\Models\SkyexDetail;
use App\Models\AccessDetail;
use Illuminate\Http\Request;
use App\Models\ShipperDetail;
use App\Models\ProductService;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;
use GuzzleHttp\Exception\ClientException;

class DetailsController extends Controller
{
    
    private function createClient($session)
    {
        $shop = $session->getShop();
        $accessToken = $session->getAccessToken();
        $baseUri = "https://{$shop}/admin/api/2023-01/";

        return new Client([
            'base_uri' => $baseUri,
            'headers' => [
                'Content-Type' => 'application/json',
                'X-Shopify-Access-Token' => $accessToken,
            ],
        ]);
    }
  
    public function storeAccessDetails(Request $request)
    {
      
        $accessDetail = new AccessDetail();
        $token= $accessDetail->api_key = $request-> apiKey;
        
      
            $response = Http::withHeaders([
                'token' =>$token,
                'Accept' => 'application/json',
            ])->get('https://ferointegration.feroai.com/api/v1/integration/product/');
        
           
            if ($response->successful())
            {
                $session = $request->get('shopifySession');
                $client = $this->createClient($session);
                $response = $client->get('shop.json');
                $domain = $session->getShop();
                $shop = DB::table('sessions')->where('shop', $domain)->first();
                
        
                $shop_details = json_decode($response->getBody(), true);
                $domain_name = $shop_details['shop']['domain'];
                //dd($domain_name,$domain);

                $accessDetail = new AccessDetail();
                $accessDetail->api_key = $request->apiKey;
                $accessDetail->customer_code =  $request-> customerCode;
                $accessDetail->shop_id = $shop->id;
                $accessDetail->domain_name = $domain_name;
              
                $accessDetail->save();
                //$productCodes = $this->getProductListApi($accessDetail->id);
                //$serviceCodes= $this->getServiceListApi($accessDetail->id);

                return response()->json(['message' => 'Data added successfully'], 200);

            }
            else {
                $errorMessage = $response->json()['errors'][0]['error_message'] ?? 'Unknown error occurred.';
                return response()->json(['error' => $errorMessage], $response->status());
            }
    }
    public function checkBackendStatus(Request $request)
{
    try {
        // Ensure 'shopifySession' exists in the request
        $session = $request->get('shopifySession');
        if (!$session) {
            return response()->json(['error' => 'Shopify session not provided'], 400);
        }

        // Create the Shopify client
        $client = $this->createClient($session);

        // Make the request to Shopify
        $response = $client->get('shop.json');
        $shop_details = json_decode($response->getBody(), true);

        // Get the shop domain and retrieve the shop record from the database
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
        $apiStatusDetails = $shop -> api_status;

        if (!$shop) {
            return response()->json(['error' => 'Shop not found in the database'], 404);
        }

        // Get the shop ID and retrieve user details from AccessDetail
        $shop_id = $shop->id;
        $userDetails = AccessDetail::where('shop_id', $shop_id)->first();
        //$skyexUserDetails = SkyexDetail::where('shop_id', $shop_id)->where('status',1)->first();
        //dd($userDetails);
         if ($userDetails) {
            $status = $userDetails->status;
        

            $data = [
                'status' => $status,
               // 'skyexStatus' => $skyexStatus,
               
                ];
            return response()->json($data, 200);
        } else {
            return response()->json('not found', 200);
        }
    } catch (\Exception $e) {
        // Handle any unexpected exceptions
        return response()->json(['error' => 'An error occurred', 'message' => $e->getMessage()], 500);
    }
}

    public function getAccessDetails(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);

        $response = $client->get('shop.json');
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();

        $shop_id = $shop->id;

        $userDetails = AccessDetail::where('shop_id', $shop_id)->first();
        $shipperDetails=ShipperDetail::where('shop_id', $shop_id)->where('is_default',1)->first();
        if($userDetails)
        {
        $productCodes = $this->getProductListApi($userDetails->id);
        $serviceCodes= $this->getServiceListApi($userDetails->id);
       
        foreach ($productCodes as $code) {
        // Check if the product_code already exists in ProductType
        $existingProduct = ProductType::where('product_type_SF', $code['product_code'])->first();

        if (!$existingProduct) {
            // If product_code doesn't exist, create a new ProductType instance and save it
            $productCodeDetail = new ProductType();
            $productCodeDetail->product_type_SF = $code['product_code'];
            $productCodeDetail->product_type_FF = $code['product_name'];
            $productCodeDetail->save();
        }
        else{
            $productCodeDetail = new ProductType();
            $productCodeDetail->product_type_SF = $code['product_code'];
            $productCodeDetail->product_type_FF = $code['product_name'];
            $productCodeDetail->update();
        }
    }


    foreach ($serviceCodes as $code) {
        // Check if the product_code already exists in ProductType
        $existingProduct = ProductService::where('product_service_SF', $code['code'])->first();

        if (!$existingProduct) {
            // If product_code doesn't exist, create a new ProductType instance and save it
            $productCodeDetail = new ProductService();
            $productCodeDetail->product_service_SF = $code['code'];
            $productCodeDetail->product_service_FF = $code['name'];
            $productCodeDetail->save();
        }
        else{
            $productCodeDetail = new ProductService();
            $productCodeDetail->product_service_SF = $code['code'];
            $productCodeDetail->product_service_FF = $code['name'];
            $productCodeDetail->update();
        }
    }
} 
   

    $data = [
        'details' => $userDetails,
       'shipperDetails'=> $shipperDetails??null
    ];


        

        //dd($productCodes);
        return response($data, 200);
    }

    public function updateDetails(Request $request)
    {
        $accessDetail = new AccessDetail();
        $token= $accessDetail->api_key = $request-> apiKey;
        //dd($token);
      
            $response = Http::withHeaders([
                'token' =>$token,
                'Accept' => 'application/json',
            ])->get('https://ferointegration.feroai.com/api/v1/integration/product/');
        
           $response_body = $response->json();

           Log::info($response_body);
            if ($response->successful())
            {

        $session = $request->get('shopifySession');
        $client = $this->createClient($session);

        $response = $client->get('shop.json');
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();

        $shop_details = json_decode($response->getBody(), true);
        $domain_name = $shop_details['shop']['domain'];
        //dd($domain_name,$domain);

        $shop_id = $shop->id;
       
        $apiKey = $request->input('apiKey');
        $customerCode = $request->input('customerCode');

       
          
            $user = AccessDetail::where('shop_id', $shop_id)->first();
            
            if ($user) {
                $user->update([
                    'customer_code' => $customerCode,
                    'api_key' => $apiKey,
                    'domain_name' => $domain_name,
                ]);
                return response()->json(['message' => 'Details updated successfully']);
            } else {
                return response()->json(['error' => 'User not found'], 404);
            }
       

    }
        else {
            $errorMessage = $response->json()['errors'][0]['error_message'] ?? 'Unknown error occurred.';
            return response()->json(['error' => $errorMessage], $response->status());
        }
        
    }
    public function submitShippingDetails(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
        
        $response = $client->get('shop.json');
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
        
        $shop_id = $shop->id;
        
        $checkedDomestic = $request->input('checkedDomestic');
        $checkedExport = $request->input('checkedExport');
        
        $selectedTime = $request->input('selectedTime');
        $readyAtTime = $request->input('readyAtTime');
        $closingTime = $request->input('closingTime');
        $weight = $request->input('weight');
        $selectedPdfSize = $request->input('selectedPdfSize');
    
        try {
            $user = AccessDetail::where('shop_id', $shop_id)->first();
            
            if ($user) {
                $updateData = [
                    'time_slot' => $selectedTime,
                    'ready_at_time' => $readyAtTime,
                    'closing_time' => $closingTime,
                    'weight' => $weight,
                    'pdf_size' => $selectedPdfSize,
                ];
    
                if ($checkedDomestic) {
                    $updateData['product_group1'] = $request->input('productGroup1');
                    $updateData['product_type1'] = $request->input('productType1');
                    $updateData['services1'] = $request->input('service1');
                } else {
                    $updateData['product_group1'] = null;
                    $updateData['product_type1'] = null;
                    $updateData['services1'] = null;
                }
    
                if ($checkedExport) {
                    $updateData['product_group2'] = $request->input('productGroup2');
                    $updateData['product_type2'] = $request->input('productType2');
                    $updateData['services2'] = $request->input('service2');
                } else {
                    $updateData['product_group2'] = null;
                    $updateData['product_type2'] = null;
                    $updateData['services2'] = null;
                }
    
                $user->update($updateData);
    
                return response()->json(['message' => 'Details updated successfully']);
            } else {
                return response()->json(['error' => 'User not found'], 404);
            }
        } catch (\Throwable $th) {
            return response()->json(['error' => 'Error occurred while updating details'], 500);
        }
    }
    public function getCountries(Request $request)
    {

        $countries  = Country::all();
        
        $data = [
            'details' => $countries,

        ];
        //dd($data);
        return response($data, 200);
    }

    public function sendCountries(Request $request)
    {
        $countryName = $request->input('country');
    
        // Initialize an empty array for cities
        $data = [
            'cities' => [],
        ];
    
        if ($countryName) {
            $country = Country::where('country_code', $countryName)->first();
    
            if ($country && $country->cities) {
                $data['cities'] = $country->cities;
            }
        }
    
       
        return response()->json($data, 200);
    }

    public function getCompanies(Request $request)
    {
    $session = $request->get('shopifySession');
    $client = $this->createClient($session);

    $response = $client->get('shop.json');
    $domain = $session->getShop();
    $shop = DB::table('sessions')->where('shop', $domain)->first();

    $shop_id = $shop->id;
    $companies = ShipperDetail::where('shop_id', $shop_id)->get();

    $companyDetails = $companies->map(function ($company) {
        return [
            'id' => $company->id, 
            'company_name' => $company->company_name,
            'primary_contact_person' => $company->primary_contact_person,
            'primary_contact_number_code' => $company->primary_contact_number_code,
            'primary_contact_number' => $company->primary_contact_number,
            'secondary_contact_person' => $company->secondary_contact_person,
            'secondary_contact_number_code' => $company->secondary_contact_number_code,
            'secondary_contact_number' => $company->secondary_contact_number,
            'address_type' => $company->address_type,
            'country' => $company->country,
            'city' => $company->city,
            'landmark' => $company->landmark,
            'zipcode' => $company->zipcode,
            'address_line' => $company->address_line,
            'is_default'=>$company->is_default
                   ];
    });

    return response()->json(['details' => $companyDetails], 200);
}
    public function submitAddressDetails(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);

        $response = $client->get('shop.json');
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();

        $shop_id = $shop->id;
      
        $validatedData = $request->validate([
            'companyId' => 'required|string',
            'company_name' =>'required|string',
            'primaryContactPerson' => 'required|string',
            'primaryContactNumber' => 'required|string',
            'primaryCode' => 'required|string',
            'secondaryContactPerson' => 'nullable|string',
            'secondaryCode' => 'nullable|string',
            'secondaryContactNumber' => 'nullable|string',
            'landmark' => 'required|string',
            'zipcode' => 'nullable|string',
            'addressLine' => 'required|string',
            'addressType' => 'nullable|string',
            'country' => 'required|string',
            'city' => 'required|string',
            'isDefaultAddressShipper' => 'nullable|boolean',
        ]);
     
        try {
            DB::beginTransaction();
            if($validatedData['isDefaultAddressShipper'])
            {
                ShipperDetail::where('shop_id', $shop_id)
                ->where('is_default', true)
                ->where('id', '!=', $validatedData['companyId']) 
                ->update(['is_default' => false]);
            $updatedShipper = ShipperDetail::where('shop_id', $shop_id)->where('id', $validatedData['companyId'])->first();
       
            $companyName = $updatedShipper->company_name;
            $user = AccessDetail::where('shop_id', $shop_id)->first();
            
            if ($user) {
                $user->update([
                    'company_name' =>  $validatedData['company_name'],
                    'primary_contact_person' => $validatedData['primaryContactPerson'],
                    'primary_contact_number' => $validatedData['primaryContactNumber'],
                    'secondary_contact_person' => $validatedData['secondaryContactPerson'],
                    'secondary_contact_number' => $validatedData['secondaryContactNumber'],
                    'landmark' => $validatedData['landmark'],
                    'zipcode' => $validatedData['zipcode'],
                    'adress_line' =>$validatedData['addressLine'],
                    'address_type' => $validatedData['addressType'],
                    'country' => $validatedData['country'],
                    'city' =>$validatedData['city'],
                    'primary_contact_number_code' =>  $validatedData['primaryCode'],
                    'secondary_contact_number_code' => $validatedData['secondaryCode'],
                    'status' => '1',
                ]);
                // $this->addApiStatus($request);
            }
           
        }
        $shipper=ShipperDetail::where('shop_id', $shop_id)
            ->where('id', $validatedData['companyId'])      
            ->update([
            'company_name' =>  $validatedData['company_name'],
            'primary_contact_person' => $validatedData['primaryContactPerson'],
            'primary_contact_number' => $validatedData['primaryContactNumber'],
            'secondary_contact_person' => $validatedData['secondaryContactPerson'],
            'secondary_contact_number' => $validatedData['secondaryContactNumber'],
            'landmark' => $validatedData['landmark'],
            'zipcode' => $validatedData['zipcode'],
            'address_line' => $validatedData['addressLine'], 
            'address_type' => $validatedData['addressType'],
            'country' => $validatedData['country'],
            'city' => $validatedData['city'],
            'primary_contact_number_code' => $validatedData['primaryCode'],
            'secondary_contact_number_code' => $validatedData['secondaryCode'],
            'is_default' => $validatedData['isDefaultAddressShipper'],
        ]);

       
        
        $default_shipper_detail = ShipperDetail::where('shop_id', $shop_id)->where('is_default', 1)->first();
        $all_shipper_details = ShipperDetail::where('shop_id', $shop_id)->get();
        
        DB::commit();
        return response()->json([
        'message' => 'Details fetched successfully',
        'default_detail' => $default_shipper_detail,
        'all_details' => $all_shipper_details
         ]);

        } catch (\Throwable $th) {
            DB::rollBack();
            return response()->json(['error' => 'Error occurred while updating details'], 500);
        }
    }

    public function saveNewShipperAddress(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
    
        $response = $client->get('shop.json');
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
    
        $shop_id = $shop->id;
        $user = AccessDetail::where('shop_id', $shop_id)->first();
        $submit_status=false;
        $refresh_status=false;
        $validatedData = $request->validate([
            'companyName' => 'required|string',
            'primaryContactPerson' => 'required|string',
            'primaryContactNumber' => 'required|string',
            'primaryCode' => 'required|string',
            'secondaryContactPerson' => 'nullable|string',
            'secondaryCode' => 'nullable|string',
            'secondaryContactNumber' => 'nullable|string',
            'landmark' => 'required|string',
            'zipcode' => 'nullable|string',
            'addressLine' => 'required|string',
            'addressType' => 'nullable|string',
            'country' => 'required|string',
            'city' => 'required|string',
            'isDefaultAddress' => 'nullable|boolean',
        ]);
    
        try {
            $shipper_first = ShipperDetail::where('shop_id', $shop_id)->first();
            $user_first=AccessDetail::where('shop_id', $shop_id)->where('delete_status',1)->first();
            if(!$shipper_first)
            {
                if($user_first)
                {
                    $submit_status=false;
                    $refresh_status=true;
                }
                else{
                    $submit_status=true;
                }
               
            }
           
           
            if($validatedData['isDefaultAddress']){

            $shipper=ShipperDetail::create([
                'shop_id' => $shop_id,
                'company_name' => $validatedData['companyName'],
                'primary_contact_person' => $validatedData['primaryContactPerson'],
                'primary_contact_number' => $validatedData['primaryContactNumber'],
                'secondary_contact_person' => $validatedData['secondaryContactPerson'],
                'secondary_contact_number' => $validatedData['secondaryContactNumber'],
                'landmark' => $validatedData['landmark'],
                'zipcode' => $validatedData['zipcode'],
                'address_line' => $validatedData['addressLine'], 
                'address_type' => $validatedData['addressType'],
                'country' => $validatedData['country'],
                'city' => $validatedData['city'],
                'primary_contact_number_code' => $validatedData['primaryCode'],
                'secondary_contact_number_code' => $validatedData['secondaryCode'],
                'is_default' => $validatedData['isDefaultAddress'],
            ]);
              
                if ($user) {
                    ShipperDetail::where('shop_id', $shop_id)
                    ->where('is_default', true)
                    ->where('id', '!=', $shipper->id) 
                    ->update(['is_default' => false]);

                    $user->update([
                        'company_name' => $validatedData['companyName'],
                        'primary_contact_person' => $validatedData['primaryContactPerson'],
                        'primary_contact_number' => $validatedData['primaryContactNumber'],
                        'secondary_contact_person' => $validatedData['secondaryContactPerson'],
                        'secondary_contact_number' => $validatedData['secondaryContactNumber'],
                        'landmark' => $validatedData['landmark'],
                        'zipcode' => $validatedData['zipcode'],
                        'adress_line' =>$validatedData['addressLine'],
                        'address_type' => $validatedData['addressType'],
                        'country' => $validatedData['country'],
                        'city' =>$validatedData['city'],
                        'primary_contact_number_code' =>  $validatedData['primaryCode'],
                        'secondary_contact_number_code' => $validatedData['secondaryCode'],
                        'status' => '1',
                    ]);
            }
            $this->addApiStatus($request);
        }
            else {
               
                $shipper = ShipperDetail::where('shop_id', $shop_id)->where('is_default',1)->first();
                
                if (! $shipper) {
                           
                            $shipper=ShipperDetail::create([
                            'shop_id' => $shop_id,
                            'company_name' => $validatedData['companyName'],
                            'primary_contact_person' => $validatedData['primaryContactPerson'],
                            'primary_contact_number' => $validatedData['primaryContactNumber'],
                            'secondary_contact_person' => $validatedData['secondaryContactPerson'],
                            'secondary_contact_number' => $validatedData['secondaryContactNumber'],
                            'landmark' => $validatedData['landmark'],
                            'zipcode' => $validatedData['zipcode'],
                            'address_line' => $validatedData['addressLine'], 
                            'address_type' => $validatedData['addressType'],
                            'country' => $validatedData['country'],
                            'city' => $validatedData['city'],
                            'primary_contact_number_code' => $validatedData['primaryCode'],
                            'secondary_contact_number_code' => $validatedData['secondaryCode'],
                            'is_default' => true,
                        ]);
                    }
                    else{
                        $shipper=ShipperDetail::create([
                            'shop_id' => $shop_id,
                            'company_name' => $validatedData['companyName'],
                            'primary_contact_person' => $validatedData['primaryContactPerson'],
                            'primary_contact_number' => $validatedData['primaryContactNumber'],
                            'secondary_contact_person' => $validatedData['secondaryContactPerson'],
                            'secondary_contact_number' => $validatedData['secondaryContactNumber'],
                            'landmark' => $validatedData['landmark'],
                            'zipcode' => $validatedData['zipcode'],
                            'address_line' => $validatedData['addressLine'], 
                            'address_type' => $validatedData['addressType'],
                            'country' => $validatedData['country'],
                            'city' => $validatedData['city'],
                            'primary_contact_number_code' => $validatedData['primaryCode'],
                            'secondary_contact_number_code' => $validatedData['secondaryCode'],
                            'is_default' => $validatedData['isDefaultAddress'],
                        ]);
                    }
                                                 
                    if ($user) {
                        $user->update([
                            'company_name' => $validatedData['companyName'],
                            'primary_contact_person' => $validatedData['primaryContactPerson'],
                            'primary_contact_number' => $validatedData['primaryContactNumber'],
                            'secondary_contact_person' => $validatedData['secondaryContactPerson'],
                            'secondary_contact_number' => $validatedData['secondaryContactNumber'],
                            'landmark' => $validatedData['landmark'],
                            'zipcode' => $validatedData['zipcode'],
                            'adress_line' => $validatedData['addressLine'],
                            'address_type' => $validatedData['addressType'],
                            'country' => $validatedData['country'],
                            'city' => $validatedData['city'],
                            'primary_contact_number_code' => $validatedData['primaryCode'],
                            'secondary_contact_number_code' => $validatedData['secondaryCode'],
                            'status' => '1',
                           
                        ]);
                    }
                    $this->addApiStatus($request);
                         
                }
            return response()->json(['message' => 'Details saved successfully','data' =>  $shipper,'submit_status'=>$submit_status,'refresh_status'=>$refresh_status]);
        } catch (\Throwable $th) {
            return response()->json(['error' => 'Error occurred while saving details'], 500);
        }
    }
    
    public function getProductType(Request $request)
    {

        $productTypes  = ProductType::all();
        
        $data = [
            'details' => $productTypes,

        ];
        //dd($data);
        return response($data, 200);
    }

    public function getProductService(Request $request)
    {

        $productServices  = ProductService::all();
        
        $data = [
            'details' => $productServices,

        ];
        //dd($data);
        return response($data, 200);
    }

    public function getPdfSize(Request $request)
    {

        $pdfSizes  = PdfSize::all();
        
        $data = [
            'details' => $pdfSizes->map(function ($size) {
                return ['size' => $size->size];
            })->toArray(),
        ];
        return response($data, 200);
    }
    private function getProductListApi($id)
    {
        $accessDetail=AccessDetail::find($id);

        $token=$accessDetail->api_key;
       
    try {
        $response = Http::withHeaders([
            'token' =>$token,
            'Accept' => 'application/json',
        ])->get('https://ferointegration.feroai.com/api/v1/integration/product/');
    
       
        if ($response->successful()) {
          
            return $response->json()['detail'];
        } else {
            $errorMessage = $response->json()['errors'][0]['error_message'] ?? 'Unknown error occurred.';
            return response()->json(['error' => $errorMessage], $response->status());
        }
    } catch (\Exception $e) {
        return response()->json(['error' => $e->getMessage()], 500);
    }
    }
    private function getServiceListApi($id)
    {
        $accessDetail=AccessDetail::find($id);

        $token=$accessDetail->api_key;
       
    try {
        $response = Http::withHeaders([
            'token' =>$token,
            'Accept' => 'application/json',
        ])->get('https://ferointegration.feroai.com/api/v1/integration/service/');
    
       
        if ($response->successful()) {
            return $response->json()['detail'];
        } else {
            $errorMessage = $response->json()['errors'][0]['error_message'] ?? 'Unknown error occurred.';
            return response()->json(['error' => $errorMessage], $response->status());
        }
    } catch (\Exception $e) {
        return response()->json(['error' => $e->getMessage()], 500);
    }
    }
    public function deleteCompany(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
    
        $response = $client->get('shop.json');
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
    
        $shop_id = $shop->id;
        $companyId = $request->input('selectedCompanyId');
    
        $company = ShipperDetail::where('shop_id', $shop_id)->where('id', $companyId)->first();
    
        if ($company) {
            if ($company->is_default) {
                $firstCompany = ShipperDetail::where('shop_id', $shop_id)
                    ->where('id', '<>', $companyId)
                    ->first();
    
                if ($firstCompany) {
                    $firstCompany->update(['is_default' => true]);
                }
            }
    
            $company->delete();
    
            $remainingCompanies = ShipperDetail::where('shop_id', $shop_id)->get();
            $defaultCompany = $remainingCompanies->firstWhere('is_default', true);
    
            if (!$defaultCompany && $remainingCompanies->isNotEmpty()) {
                $firstCompany = $remainingCompanies->first();
                $firstCompany->update(['is_default' => true]);
                $defaultCompany = $firstCompany;
            }
    
            $user = AccessDetail::where('shop_id', $shop_id)->first();
          
            if ($remainingCompanies->isEmpty()) {
                if ($user) {
                    
                    $user->update([
                        'company_name' => null,
                        'primary_contact_person' => null,
                        'primary_contact_number' => null,
                        'secondary_contact_person' => null,
                        'secondary_contact_number' => null,
                        'landmark' => null,
                        'zipcode' => null,
                        'adress_line' => null,
                        'address_type' => null,
                        'country' => null,
                        'city' => null,
                        'primary_contact_number_code' => null,
                        'secondary_contact_number_code' => null,
                        'status' => '0',
                        'delete_status' => true, 
                    ]);
                }
            } else if ($defaultCompany) {
                if ($user) {
                    $user->update([
                        'company_name' => $defaultCompany->company_name,
                        'primary_contact_person' => $defaultCompany->primary_contact_person,
                        'primary_contact_number' => $defaultCompany->primary_contact_number,
                        'secondary_contact_person' => $defaultCompany->secondary_contact_person,
                        'secondary_contact_number' => $defaultCompany->secondary_contact_number,
                        'landmark' => $defaultCompany->landmark,
                        'zipcode' => $defaultCompany->zipcode,
                        'adress_line' => $defaultCompany->address_line,
                        'address_type' => $defaultCompany->address_type,
                        'country' => $defaultCompany->country,
                        'city' => $defaultCompany->city,
                        'primary_contact_number_code' => $defaultCompany->primary_contact_number_code,
                        'secondary_contact_number_code' => $defaultCompany->secondary_contact_number_code,
                        'status' => '1',
                        'delete_status' => false, 
                    ]);
                }
            }
    
            return response()->json([
                'success' => true,
                'message' => 'Company deleted successfully',
                'data' => [
                    'remainingCompanies' => $remainingCompanies,
                    'defaultCompany' => $defaultCompany,
                    ],
            ]);
        } else {
            return response()->json(['success' => false, 'message' => 'Company not found'], 404);
        }
    }
    

    public function addApiStatus(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
        $response = $client->get('shop.json');
        $domain = $session->getShop();
    
        DB::table('sessions')
            ->where('shop', $domain)
            ->update(['api_status' => 1]);
    
        //$shop = DB::table('sessions')->where('shop', $domain)->first();
        //dd($shop->api_status); // This will display the updated api_status
    }
    public function changeApiStatus(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
        $response = $client->get('shop.json');
        $domain = $session->getShop();
    
        DB::table('sessions')
            ->where('shop', $domain)
            ->update(['api_status' => null]);
    
        //$shop = DB::table('sessions')->where('shop', $domain)->first();
        //dd($shop->api_status); // This will display the updated api_status
    }
    
    public function buttonChangeApiStatus(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
        $response = $client->get('shop.json');
        $domain = $session->getShop();
    
        DB::table('sessions')
            ->where('shop', $domain)
            ->update(['buttonChangeApiStatus' => $request->buttonChangeApiStatus]);
    
           return response()->json(['success' => true]);
        //$shop = DB::table('sessions')->where('shop', $domain)->first();
        //dd($shop->api_status); // This will display the updated api_status
    }


//     public function buttonChangeApiStatus(Request $request)
// {
//     try {
//         $session = $request->get('shopifySession');
//         if (!$session) {
//             return response()->json(['error' => 'No Shopify session found'], 400);
//         }

//         $client = $this->createClient($session);
//         $response = $client->get('shop.json');
//         $domain = $session->getShop();
    
//         DB::table('sessions')
//             ->where('shop', $domain)
//             ->update(['buttonChangeApiStatus' => $request->buttonChangeApiStatus]);
    
//         return response()->json(['success' => true]);
//     } catch (\Exception $e) {
//         return response()->json(['error' => $e->getMessage()], 500);
//     }
// }
}
