<?php

namespace App\Http\Controllers;

use App\Models\Order;
use GuzzleHttp\Client;
use App\Models\Country;
use App\Models\Session;
use setasign\Fpdi\Fpdi;
use App\Jobs\FulfillmentJob;
use App\Models\AccessDetail;
use Illuminate\Http\Request;
use App\Models\ShipperDetail;
use App\Models\ReturnOrderDetail;
use App\Models\SkyexDetail;
use Illuminate\Support\Facades\DB;

use Illuminate\Support\Facades\Log;
use libphonenumber\PhoneNumberType;
use libphonenumber\PhoneNumberUtil;
use Illuminate\Support\Facades\Http;
use libphonenumber\PhoneNumberFormat;
use Illuminate\Support\Facades\Storage;
use libphonenumber\NumberParseException;
use GuzzleHttp\Exception\ClientException;

class OrderController extends Controller
{
    private function createClient($shop, $accessToken)
    {
    
        $baseUri = "https://{$shop}/admin/api/2023-04/";
    
        return new Client([
            'base_uri' => $baseUri,
            'headers' => [
                'Content-Type' => 'application/json',
                'X-Shopify-Access-Token' => $accessToken,
            ],
        ]);
    }
    public function getOrders(Request $request)
    {
         
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

        if (!$shop) {
            return response()->json(['error' => 'Session not found'], 404);
        }
        $client = $this->createClient($domain, $accessToken);

        try {
            $accessDetail = $shop->accessDetail;
            $shop_id = $shop->id;
            $accessDetail = AccessDetail::where('shop_id', $shop_id)->first();
            $skyexDetail  = SkyexDetail::where('shop_id', $shop_id)->first();
            //dd($accessSkyExDetail);
            //dd($shop->accessDetail);

             // Update existing orders if both details are available
             if ($accessDetail && $skyexDetail) {
                Order::where('skyex_access_id', $skyexDetail->id)
                    ->whereNull('access_id')
                    ->update(['access_id' => $accessDetail->id]);
            }


            
            $orders = [];
            $response = $client->get("orders.json");
            $result = json_decode($response->getBody(), true);
        
            $existingOrderIds = Order::pluck('order_id')->toArray();
        
            foreach ($result['orders'] as $order) {
                if (in_array($order['id'], $existingOrderIds)) {
                    continue;
                }
        
                // $isFulfilled = false;
                //   if ($order['fulfillment_status'] == 'fulfilled') {
                //     $isFulfilled = true;
                //     break;
                // }
               
                $variantData = [];
        
                foreach ($order['line_items'] as $item) {
                   
                    $productId = $item['product_id'];
                    if($productId)
                    {
                    $response = $client->get("products/$productId.json");
                    $productResult = json_decode($response->getBody(), true);
        
                    foreach ($productResult['product']['variants'] as $variant) {
                        if($item['variant_id']==$variant['id'])
                        {
                        $product_id = $productResult['product']['id'];
                        $inventoryItemId = $variant['inventory_item_id'];
                        $inventoryResponse = $client->get("inventory_items/$inventoryItemId.json");
                        $inventoryItem = json_decode($inventoryResponse->getBody(), true);
        
                        $variantData[] = [
                            'product_id' => $product_id,
                            'harmonized_system_code' => $inventoryItem['inventory_item']['harmonized_system_code'] ?? null,
                        ];
                    }
                    }
                }
                }
    

                 
                if ($order['fulfillment_status'] == null) {
                 //   if (!$isFulfilled) {
                    $order_data = [
                        'order_id' => $order['id'],
                        'order_no'=>$order['order_number'],
                        // 'access_id' => $accessDetail->id,
                        'access_id' => $accessDetail ? $accessDetail->id : null,
                        // 'skyex_access_id' => null,
                        'skyex_access_id' => $skyexDetail ? $skyexDetail->id : null,
                        'order_data' => $order,
                        'line_items' => $order['line_items'],
                        'hs_codes' => $variantData, 
                        'status' => 0, 
                    ];
        
                    Order::create($order_data);
                }
            }
        
   
            $orders = Order::
            // where('status', 0)
            where('access_id', $accessDetail->id)
                ->where('status', 0)
                ->where('remove_status', 0)
                ->orderBy('order_no', 'desc') 
                ->orderBy('created_at', 'desc')
                ->get();
    
            $pushed_orders = Order::where('access_id', $accessDetail->id)
            ->where('status', 1)
            ->whereNotNull('awb_no')
            ->orderBy('order_no', 'desc') 
            ->orderBy('created_at', 'desc')
            ->get();

            $removed_orders=Order::where('access_id', $accessDetail->id)
            ->where('status', 0)
            ->where('remove_status', 1)
            ->orderBy('order_no', 'desc') 
            ->orderBy('created_at', 'desc')
            ->get();
                
            $returned_orders=ReturnOrderDetail::where('access_id', $accessDetail->id)
            ->orderBy('created_at', 'desc')
            ->get();

        return response()->json([
            'orders' => $orders,
            'pushed_orders'=>$pushed_orders,
            'removed_orders'=>$removed_orders,
            'returned_orders'=>$returned_orders
            ], 200);
                }catch (ClientException $e) {
       
                    Log::error("Error fulfilling order: " . $e->getMessage());
                    return [
                        'success' => false,
                        'message' => "Error fulfilling order: " . $e->getMessage(),
                    ];
                }
    }

    public function getAWBNumber(Request $request)
    {
        $orderId = $request->input('orderId');
        $selectedCompanyId=$request->input('selectedCompanyId');
      
        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
                       ->where('access_token', $accessToken)
                       ->first();
    
        if (!$shop) {
            return response()->json(['error' => 'Session not found'], 404);
        }
        
        $accessDetail = $shop->accessDetail;
       //dd($shop->accessDetail);
    $shipper_details=ShipperDetail::where('shop_id',$shop->id)->where('id',$selectedCompanyId)->first();

        $order = Order::where('access_id', $accessDetail->id)
                      ->where('id', $orderId)
                      ->first();
    
        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }
    
        try {
            $awb_number = $this->getShipmentDetails($accessDetail->id, $order,$shipper_details);
    
            $pushed_orders = Order::where('access_id', $accessDetail->id)
                                  ->where('status', 1)
                                  ->whereNotNull('awb_no')
                                  ->orderBy('order_no', 'desc') 
                                  ->orderBy('created_at', 'desc')
                                  ->get();
            
            $orders = Order::where('access_id', $accessDetail->id)
                           ->where('status', 0)
                           ->where('remove_status', 0)
                           ->orderBy('order_no', 'desc') 
                           ->orderBy('created_at', 'desc')
                           ->get();
    
            $removed_orders = Order::where('access_id', $accessDetail->id)
                                  ->where('status', 0)
                                  ->where('remove_status', 1)
                                  ->orderBy('order_no', 'desc') 
                                  ->orderBy('created_at', 'desc')
                                  ->get();
    
            return response()->json(['orders' => $orders, 'pushed_orders' => $pushed_orders, 'removed_orders' => $removed_orders], 200);
        } catch (\Exception $e) {
            
            Log::error('Error in getAWBNumber: ' . $e->getMessage());
    
            // Retrieve current lists of orders
            $orders = Order::where('access_id', $accessDetail->id)
                           ->where('status', 0)
                           ->where('remove_status', 0)
                           ->orderBy('order_no', 'desc') 
                           ->orderBy('created_at', 'desc')
                           ->get();
    
            $pushed_orders = Order::where('access_id', $accessDetail->id)
                                  ->where('status', 1)
                                  ->whereNotNull('awb_no')
                                  ->orderBy('order_no', 'desc') 
                                  ->orderBy('created_at', 'desc')
                                  ->get();
    
            $removed_orders = Order::where('access_id', $accessDetail->id)
                                   ->where('status', 0)
                                   ->where('remove_status', 1)
                                   ->orderBy('order_no', 'desc') 
                                   ->orderBy('created_at', 'desc')
                                   ->get();
    
            return response()->json([
                'error' => $e->getMessage(),
                'orders' => $orders,
                'pushed_orders' => $pushed_orders,
                'removed_orders' => $removed_orders
            ], 500);
        }
    }

    private function getShipmentDetails($id,$order,$shipper_details)
    {
        $accessDetail = AccessDetail::find($id);
        $session = $accessDetail->sessions;
      
          
            if (!$accessDetail) {
                throw new \Exception('AccessDetail not found');
            }

            $token = $accessDetail->api_key;
            $customer_code = $accessDetail->customer_code;

            $defaultWeight=$accessDetail->weight;

            $order_data=$order->order_data;
            if (!$order || !$order_data) {
                throw new \Exception('Order data not found or empty');
            }

            $shipping_address = $order_data->shipping_address;

            if (!$shipping_address) {
                throw new \Exception('Shipping address not found or empty');
            }

            $order_items = [];
            $order_packages = [];
            $line_items = $order->line_items;
            $hs_codes=$order->hs_codes;
            $refunds= $order_data->refunds;
            $fulfillments= $order_data->fulfillments;
            $refunded_line_item_ids = [];
            if($refunds)
            {
            foreach ($refunds as $refund) {
                foreach ($refund->refund_line_items as $refunded_line_item) {
                    $refunded_line_item_ids[] = $refunded_line_item->line_item_id;
                }
            }
            }
            $fulfillment_line_item_ids = [];
            if($fulfillments)
            {
            foreach ($fulfillments as $fulfillment) {
                foreach ($fulfillment->line_items as $fulfillment_line_item) {
                    $fulfillment_line_item_ids[] = $fulfillment_line_item->id;
                }
            }
            }
            $total_weight = 0;
            $contents = [];

            foreach ($line_items as $line_item) {
            
                if (in_array($line_item->id, $refunded_line_item_ids) || in_array($line_item->id, $fulfillment_line_item_ids)) {
                    continue;
                }
                
            
                if (empty($line_item->fulfillable_quantity) || empty($line_item->price) || empty($line_item->name)) {
                    throw new \Exception('Product data incomplete');
                }
                
            
                $assigned_hscode = "00000000";
                foreach ($hs_codes as $hs_code) {
                    if ($hs_code->product_id == $line_item->product_id) {
                        $assigned_hscode = $hs_code->harmonized_system_code ?? "00000000";
                        break;
                    }
                }
                
            
                if (empty($assigned_hscode)) {
                    throw new \Exception('HS code is wrong');
                }
                
                
                $weight = round((float)$line_item->grams / 1000, 2);
                $total_weight += $weight;
                
                
                $order_items[] = [
                    "weight" => $weight,
                    "qty" => (int)$line_item->fulfillable_quantity,
                    "price" => (float)$line_item->price,
                    "is_dangerous" => false,
                    "assigned_hscode" => (string)$assigned_hscode,
                    "content" => $line_item->name
                ];
                
                
                $contents[] = $line_item->name;
            }
            if ($total_weight == 0) {
                $total_weight = $defaultWeight;
            }

            $concatenated_content = implode(", ", $contents);


            $order_packages[] = [
                "barcode_value" => "",
                "content" => $concatenated_content,
                "height" => 1,
                "length" => 1,
                "weight" => round((float)($total_weight), 2),
                "width" => 1
            ];

            if($order->awb_no)
            {
                $awb_no=$order->awb_no;
            }
            else{
                $awb_no="";
            }
            if($order_data->note)
            {
                $special_instruction=$order_data->note;
            }
            else{
                $special_instruction="No";
            }

            $closing_time = $accessDetail->closing_time;
            $ready_at_time = $accessDetail->ready_at_time;
            $payment_gateway=$order_data->payment_gateway_names;

            $payment_terms=$order_data->payment_terms;

            $cod_value = 0.0; 

            if (empty($payment_gateway) && $payment_terms) {
                $cod_value = (float)$order_data->current_total_price;
            
            }

            if (is_array($payment_gateway) && in_array("Cash on Delivery (COD)", $payment_gateway)) {
                $cod_value = (float)$order_data->current_total_price;

            }


            if (empty($shipping_address->address1) ||  empty($shipping_address->country_code)) {
                throw new \Exception('Consignee information incomplete');
            }
            if(empty($shipping_address->phone)){
                throw new \Exception('Consignee phone number is empty');
            }
            if(empty($shipping_address->first_name)){
                throw new \Exception('Consignee name is empty');
            }
            if(empty($shipping_address->city)){
                throw new \Exception('Consignee city is empty');
            }
            $countryCode = $shipping_address->country_code;

            if($countryCode=="AE")
            {
            $service_type = $accessDetail->product_group1;
            $product = $accessDetail->product_type1;
            $service = $accessDetail->services1;
            }
            else{
                $service_type ="Export";
                $product = $accessDetail->product_type2;
                $service = $accessDetail->services2;
            }

            $phoneUtil = PhoneNumberUtil::getInstance();
                
                try {
                    $exampleNumber = $phoneUtil->getExampleNumberForType($countryCode, PhoneNumberType::MOBILE);
                
                    $countryCallingCode =(string)$exampleNumber->getCountryCode();
                
                    
                } catch (NumberParseException $e) {
                    throw new \Exception('Phone number invalid');
                }

            
                $address= $shipping_address->address1;
                
            if (!empty($shipping_address->address2)) {
                    $address .= ', Apartment: ' . $shipping_address->address2;
                }
                if (!empty($shipping_address->city) && $shipping_address->city != '-') {
                    $address .= ', ' . $shipping_address->city;
                }
                $city=$shipping_address->city;

                if (!empty($shipping_address->city) && $shipping_address->city != '-') {
                    
                    $city = $shipping_address->city;
                
                } 
                else{
                    $city =$shipping_address->province;
                }
            
            
                $landmark = $shipping_address->address1;
            
                if (!empty($city) && $city != '-') {
                    $landmark .= ', ' . $city;
                } elseif (!empty($shipping_address->province) && $shipping_address->province != '-') {
                    $landmark .= ', ' . $shipping_address->province;
                }

                if($shipping_address->company)
                {
                    $company_name=$shipping_address->company;
                }
                else{
                    $company_name="N/A";
                }

                if ($shipping_address->first_name || $shipping_address->last_name) {
                    if ($shipping_address->first_name && $shipping_address->last_name) {
                        $primary_contact_person = $shipping_address->first_name . ' ' . $shipping_address->last_name;
                    } elseif ($shipping_address->first_name) {
                        $primary_contact_person = $shipping_address->first_name;
                    } else {
                        $primary_contact_person = $shipping_address->last_name;
                    }
                }
            

                $consignee_information = [
                "address" =>  $address,
                "city" => $city,
                "company_name" => $company_name,
                "landmark" => $landmark,
                "phone_code" =>$countryCallingCode,
                "primary_contact_number" => $shipping_address->phone,
                "primary_contact_person" => $primary_contact_person,
                "secondary_contact_number" => "",
                "secondary_contact_person" => "",
                "postcode" => $shipping_address->zip ?? "",
                "country" => $shipping_address->country_code
            ];


            //$consignment_value = 49;
            $insurance_value = 1;
            $is_reverse_shipment = false;

            if($shipper_details)
            {
                $shipper_landMark= $shipper_details->landmark;
            
                if (!empty($shipper_details->city) && $shipper_details->city != '-') {
                    $shipper_landMark .= ', ' . $shipper_details->city;
                } 
                    $shipper_information = [
                    "address" =>$shipper_details->address_line,
                    "city" =>$shipper_details->city,
                    "company_name" => $shipper_details->company_name,
                    "landmark" => $shipper_landMark,
                    "phone_code" => $shipper_details->primary_contact_number_code,
                    "primary_contact_number" => $shipper_details->primary_contact_number,
                    "primary_contact_person" => $shipper_details->primary_contact_person,
                    "secondary_contact_number" => $shipper_details->secondary_contact_number??"",
                    "secondary_contact_person" => $shipper_details->secondary_contact_person??"",
                    "country" => $shipper_details->country,
                    "postcode" => $shipper_details->zipcode??""
                ];
            }
            else{
                $shipper_landMark= $accessDetail->landmark;
            
            if (!empty($accessDetail->city) && $accessDetail->city != '-') {
                $shipper_landMark .= ', ' . $accessDetail->city;
            } 
            $shipper_information = [
                "address" => $accessDetail->adress_line,
                "city" => $accessDetail->city,
                "company_name" => $accessDetail->company_name,
                "landmark" => $shipper_landMark,
                "phone_code" => $accessDetail->primary_contact_number_code,
                "primary_contact_number" => $accessDetail->primary_contact_number,
                "primary_contact_person" => $accessDetail->primary_contact_person,
                "secondary_contact_number" => $accessDetail->secondary_contact_number??"",
                "secondary_contact_person" => $accessDetail->secondary_contact_person??"",
                "country" => $accessDetail->country,
                "postcode" => $accessDetail->zipcode??""
            ];
            }

            try {
                $response = Http::withHeaders([
                    'token' => $token,
                    'Accept' => 'application/json',
                ])->post('https://ferointegration.feroai.com/api/v1/integration/shipment/', [
                    "closing_time" => $closing_time,
                    "cod_value" => $cod_value,
                    "service_type" =>$service_type,
                    "awb_number" =>"",
                    "consignee_information" => $consignee_information,
                    "customer" =>  $customer_code,
                    "insurance_value" => $insurance_value,
                    "is_reverse_shipment" => $is_reverse_shipment,
                    "order_items" =>  $order_items,
                    "order_packages" => $order_packages,
                    "order_reference_number" => "",
                    "product" => $product,
                    "ready_at_time" => $ready_at_time,
                    "service" => $service,
                    "shipper_information" => $shipper_information,
                    "shipper_reference_number" => "",
                    "special_instruction" => $special_instruction
                ]);

                $response_body = $response->json();

                Log::info($response_body);

                if ($response->successful()) {
                    $awb_number = $response_body['detail']['awb_number'] ?? null;
                    $awb_status=$response_body['detail']['status'] ?? null;
                    
                    Log::info($awb_number);
                
                    if ($awb_number && $awb_status !='Pending') {    
                        
                        $order->awb_no = $awb_number;
                        $order->pickup_status=$awb_status;
                        $order->status = 1;
                        $order->save();
                        $fulfillment=$this->getFulfillment($order,$session,$awb_number);
                        
                    // $maxRetries = 5; 
                    // $retryDelay = 5;
                    // $retryCount = 0;

                    // while ($retryCount < $maxRetries) {
                    //     $track_response = Http::withHeaders([
                    //         'token' => $token,
                    //         'Accept' => 'application/json',
                    //     ])->post('https://ferointegrationuat.feroai.com/api/v1/integration/awb_track/', [
                    //         "awb_numbers" => $awb_number
                    //     ]);

                    //     Log::info($track_response);
                    //     $jsonResponse = $track_response->json();

                    //     if ($jsonResponse && isset($jsonResponse['success']) && $jsonResponse['success']) {
                    //         $detail = $jsonResponse['detail'][0];
                    //         if (!empty($detail['awb_logs'])) {
                    //             $awbLogs = $detail['awb_logs'];
                    //             $awb_status = $awbLogs[0]['status'];
                    //             Log::info($awb_status);
                    //             $order->awb_no = $awb_number;
                    //             $order->pickup_status = $awb_status;
                    //             $order->status = 1;
                    //             $order->save();

                    //             if ($awb_status == 'Ready For Pickup' || $awb_status == 'Created') {
                    //                 $fulfillment = $this->getFulfillment($order, $session, $awb_number);
                    //             }
                    //             break;
                    //         }
                    //     }

                    //     $retryCount++;
                    //     if ($retryCount < $maxRetries) {
                    //         sleep($retryDelay); // Wait before retrying
                    //     } else {
                    //         return response()->json(['error' => 'AWB status not found after multiple attempts'], 500);
                    //     }
                    // }
                        
                                
                            } else {
                                $order->awb_no = $awb_number;
                                $order->pickup_status=$awb_status;
                                $order->status = 0;
                                $order->save();
                                $errorMessage=$response_body['detail']['warning']??'Order is pending';
                                throw new \Exception($errorMessage);
                                //return response()->json(['error' => 'AWB number not found in response'], 500);
                            }
                        } else {
                    $errorMessage = $response_body['errors'][1]['error_message'] ?? 'Unknown error occurred.';
                    throw new \Exception($errorMessage);
                    //return response()->json(['error' => $errorMessage], $response->status());
                }
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
                //return response()->json(['error' => $e->getMessage()], 500);
            }
    } 

function capitalizeCity($city) {
    // Trim whitespace and convert the first character of each word to uppercase
    return preg_replace_callback('/\b\w/', function($matches) {
        return mb_strtoupper($matches[0]);
    }, mb_strtolower(trim($city)));
}
private function getFulfillment($order, $session, $awb_number)
{
    $shop = $session->shop;
    $orderId = $order->order_id;
    $accessToken = $session->access_token;

    $client = $this->createClient($shop, $accessToken);

    try {
        
        $response = $client->get("orders/{$orderId}/fulfillment_orders.json");
        $result = json_decode($response->getBody(), true);
        Log::info(json_encode($result));

       
        if (isset($result['fulfillment_orders']) && !empty($result['fulfillment_orders'])) {
            $fulfillment_order = $result['fulfillment_orders'][0];
            $fulfillment_id = $fulfillment_order['id'];

           
            $response = $client->post('fulfillments.json', [
                'json' => [
                    'fulfillment' => [
                        'line_items_by_fulfillment_order' => [
                            [
                                'fulfillment_order_id' => $fulfillment_id,
                            ]
                        ],
                        'tracking_info' => [
                            'number' => $awb_number,
                        ],
                        'message' => "Your custom message here",
                        'notify_customer' => true, 
                    ]
                ]
            ]);

            
            $statusCode = $response->getStatusCode();
            
            if ($statusCode >= 200 && $statusCode <= 299) {
                return [
                    'success' => true,
                    'message' => "Fulfillment created successfully.",
                ];
            } else {
                return [
                    'success' => false,
                    'message' => "Failed to create fulfillment. Status code: {$statusCode}",
                ];
            }
        } else {
            Log::warning("No fulfillment orders found for order ID: {$orderId}");
            return [
                'success' => false,
                'message' => "No fulfillment orders found for the order.",
            ];
        }
    } catch (ClientException $e) {
       
        Log::error("Error fulfilling order: " . $e->getMessage());
        return [
            'success' => false,
            'message' => "Error fulfilling order: " . $e->getMessage(),
        ];
    } catch (\Exception $e) {
        
        Log::error("Exception: " . $e->getMessage());
        return [
            'success' => false,
            'message' => "Exception: " . $e->getMessage(),
        ];
    }
}

public function getDownloadLabel(Request $request)
{
        $id=$request->input('orderId');
        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
            ->where('access_token', $accessToken)
            ->first();

            if (!$shop) {
                return response()->json(['error' => 'Session not found'], 404);
            }

        
        $accessDetail = $shop->accessDetail;
        $pdf_size=(string)$accessDetail->pdf_size;

        $token = $accessDetail->api_key;

        $download_order = Order::where('access_id', $accessDetail->id)
        ->where('status', 1)
        ->where('id', $id)
        ->first();

        if($download_order)
        {
            $awbNo=$download_order->awb_no;
            $download_response = Http::withHeaders([
                'token' => $token,
                'Accept' => 'application/json',
            ])->post('https://ferointegration.feroai.com/api/v1/integration/awb_pdf/', [
                "awb_number" =>  $awbNo,
                "pdf_size" => $pdf_size
            
            ]);

                Log::info($download_response);
                $jsonResponse = $download_response->json();
                
            
        if ($download_response->successful() && $jsonResponse['success']) {
        
            $awbLink = $jsonResponse['detail']['awb_link'];
            $download_order->label=$awbLink;
            $download_order->save();
            return response()->json(['awb_link' => $awbLink]); 
            
        } else {
        
            if ($download_response->failed()) {
                Log::error('Failed to fetch AWB PDF: ' . $download_response->status());
            }
            return back()->with('error', 'Failed to fetch AWB PDF. Please try again later.');
        }

        }

}

public function getDownloadLabelAll(Request $request)
{
    $orderIds = $request->input('orderIds'); 
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $accessDetail = $shop->accessDetail;
    $token = $accessDetail->api_key;
    $pdf_size=(string)$accessDetail->pdf_size;

    $pdfFiles = [];

    foreach ($orderIds as $id) {
        $download_order = Order::where('access_id', $accessDetail->id)
            ->where('status', 1)
            ->where('id', $id)
            ->first();

        if ($download_order) {
            $awbNo = $download_order->awb_no;
            $download_response = Http::withHeaders([
                'token' => $token,
                'Accept' => 'application/json',
            ])->post('https://ferointegration.feroai.com/api/v1/integration/awb_pdf/', [
                "awb_number" =>  $awbNo,
                "pdf_size" => $pdf_size
            ]);

            Log::info($download_response);
            $jsonResponse = $download_response->json();

            if ($download_response->successful() && $jsonResponse['success']) {
                $awbLink = $jsonResponse['detail']['awb_link'];
                $pdfContent = file_get_contents($awbLink);
                $fileName = 'pdfs/' . uniqid() . '.pdf';
                Storage::put($fileName, $pdfContent);
                $pdfFiles[] = storage_path('app/' . $fileName);
                $download_order->label = $awbLink;
                $download_order->save();
            } else {
                if ($download_response->failed()) {
                    Log::error('Failed to fetch AWB PDF: ' . $download_response->status());
                }
                return response()->json(['error' => 'Failed to fetch AWB PDF. Please try again later.'], 500);
            }
        }
    }

    if (empty($pdfFiles)) {
        return response()->json(['error' => 'No PDFs to merge.'], 400);
    }

    $mergedPdf = $this->mergePdfs($pdfFiles);

    // Clean up the temporary PDF files
    foreach ($pdfFiles as $file) {
        @unlink($file);
    }

    return response($mergedPdf, 200)
        ->header('Content-Type', 'application/pdf')
        ->header('Content-Disposition', 'attachment; filename="merged.pdf"');
}

private function mergePdfs(array $pdfFiles)
{
    $pdf = new Fpdi();

    foreach ($pdfFiles as $file) {
        $pageCount = $pdf->setSourceFile($file);
        for ($i = 1; $i <= $pageCount; $i++) {
            $tplIdx = $pdf->importPage($i);
            $specs = $pdf->getTemplateSize($tplIdx);

            $pdf->AddPage($specs['orientation'], [$specs['width'], $specs['height']]);
            $pdf->useTemplate($tplIdx);
        }
    }

    return $pdf->Output('S');
}

    // Return all PDF links
   // return response()->json(['pdfs' => $pdfs]);


public function getRemoveAll(Request $request)
{
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $orderIds = $request->input('orderIds');

    $accessDetail = $shop->accessDetail;

    foreach ($orderIds as $id) {
        $remove_order = Order::where('access_id', $accessDetail->id)
            ->where('id', $id)
            ->first();

        if ($remove_order) {
            $remove_order->remove_status = 1;
            $remove_order->save(); 
        }
    }

    $orders = Order::where('access_id', $accessDetail->id)
        ->where('status', 0)
        ->where('remove_status', 0)
        ->orderBy('order_no', 'desc') 
        ->orderBy('created_at', 'desc')
        ->get();

    $removed_orders = Order::where('access_id', $accessDetail->id)
         ->where('status', 0)
        ->where('remove_status', 1)
        ->orderBy('created_at', 'desc')
        ->get();


    return response()->json([
        'orders' => $orders,
        'removed_orders' => $removed_orders
    ], 200);
}

public function getReloadDataAll(Request $request)
{
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $client = $this->createClient($domain, $accessToken);
    $orderIds = $request->input('orderIds', []);

    if (empty($orderIds)) {
        return response()->json(['error' => 'No order IDs provided'], 400);
    }

    try {
        $accessDetail = $shop->accessDetail;
        $orders = [];
        $response = $client->get("orders.json");
        $result = json_decode($response->getBody(), true);

        
        $ordersToUpdate = Order::whereIn('id', $orderIds)->pluck('order_id')->toArray();
     
        $updatedOrders = [];

        foreach ($result['orders'] as $order) {
            if (in_array($order['id'], $ordersToUpdate)) {
                $isFulfilled = false;
                $variantData = [];

                foreach ($order['line_items'] as $item) {
                    if ($item['fulfillment_status'] || $item['fulfillment_status'] == 'fulfilled') {
                        $isFulfilled = true;
                        break;
                    }

                    $productId = $item['product_id'];
                    $response = $client->get("products/{$productId}.json");
                    $productResult = json_decode($response->getBody(), true);

                    foreach ($productResult['product']['variants'] as $variant) {
                        if ($item['variant_id'] == $variant['id']) {
                            $product_id = $productResult['product']['id'];
                            $inventoryItemId = $variant['inventory_item_id'];
                            $inventoryResponse = $client->get("inventory_items/$inventoryItemId.json");
                            $inventoryItem = json_decode($inventoryResponse->getBody(), true);

                            $variantData[] = [
                                'product_id' => $product_id,
                                'harmonized_system_code' => $inventoryItem['inventory_item']['harmonized_system_code'] ?? null,
                            ];
                        }
                    }
                }

                if (!$isFulfilled) {
                    $order_data = [
                        'order_id' => $order['id'],
                        'access_id' => $accessDetail->id,
                        'order_data' => $order,
                        'line_items' => $order['line_items'],
                        'hs_codes' => $variantData,
                        'status' => 0,
                    ];

                    Order::where('order_id', $order['id'])
                    ->where('access_id',$accessDetail->id)->update($order_data);
                    $updatedOrders[] = $order_data;
                }
            }
        }

        return response()->json(['updatedOrders' => $updatedOrders, 'message' => 'Orders updated successfully']);
    } catch (\Exception $e) {
        return response()->json(['error' => 'An error occurred: ' . $e->getMessage()], 500);
    }
}
public function getReloadData(Request $request)
{
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $client = $this->createClient($domain, $accessToken);
    $orderId = $request->input('orderId');

    if (empty($orderId)) {
        return response()->json(['error' => 'No order ID provided'], 400);
    }

    try {
        $accessDetail = $shop->accessDetail;
        $orderToUpdate = Order::where('id', $orderId)->first();
        $order_id=$orderToUpdate->order_id;
     
        $response = $client->get("orders/$order_id.json");
        $orderResult = json_decode($response->getBody(), true);

       

        if (!$orderToUpdate) {
            return response()->json(['error' => 'Order not found in the database'], 404);
        }

        $isFulfilled = false;
        $variantData = [];

        foreach ($orderResult['order']['line_items'] as $item) {
            if ($item['fulfillment_status'] || $item['fulfillment_status'] == 'fulfilled') {
                $isFulfilled = true;
                break;
            }

            $productId = $item['product_id'];
            $productResponse = $client->get("products/$productId.json");
            $productResult = json_decode($productResponse->getBody(), true);

            foreach ($productResult['product']['variants'] as $variant) {
                if ($item['variant_id'] == $variant['id']) {
                    $product_id = $productResult['product']['id'];
                    $inventoryItemId = $variant['inventory_item_id'];
                    $inventoryResponse = $client->get("inventory_items/$inventoryItemId.json");
                    $inventoryItem = json_decode($inventoryResponse->getBody(), true);

                    $variantData[] = [
                        'product_id' => $product_id,
                        'harmonized_system_code' => $inventoryItem['inventory_item']['harmonized_system_code'] ?? null,
                    ];
                }
            }
        }

        if (!$isFulfilled) {
            $order_data = [
                'order_id' => $orderResult['order']['id'],
                'access_id' => $accessDetail->id,
                'order_data' => $orderResult['order'],
                'line_items' => $orderResult['order']['line_items'],
                'hs_codes' => $variantData,
                'status' => 0,
            ];

            Order::where('order_id', $orderResult['order']['id'])
            ->where('access_id',$accessDetail->id)->update($order_data);

            return response()->json(['updatedOrder' => $order_data, 'message' => 'Order updated successfully']);
        } else {
            return response()->json(['message' => 'Order is already fulfilled, no update necessary']);
        }
    } catch (\Exception $e) {
        return response()->json(['error' => 'An error occurred: ' . $e->getMessage()], 500);
    }
}


}
