<?php

namespace App\Http\Controllers;

use Exception;
use App\Models\Order;
use GuzzleHttp\Client;
use App\Models\Session;
use App\Models\Webhook;
use App\Models\Shipment;
use App\Models\AccessDetail;

use Illuminate\Http\Request;
use Shopify\Webhooks\Registry;
use Shopify\Clients\HttpHeaders;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use GrahamCampbell\ResultType\Success;
use GuzzleHttp\Exception\GuzzleException;
use Shopify\Exception\InvalidWebhookException;

class WebhookController extends Controller
{

    private function createClient($session)
    {
        $shop = $session->getShop();
        $accessToken = $session->getAccessToken();
        $baseUri = "https://{$shop}/admin/api/2023-01/";

        return new Client([
            'base_uri' => $baseUri,
            'headers' => [
                'Content-Type' => 'application/json',
                'X-Shopify-Access-Token' => $accessToken,
            ],
        ]);
    }
    protected function verifyShopifyWebhook(Request $request)
    {
        $hmacHeader = $request->header('X-Shopify-Hmac-SHA256', '');
        if (!$hmacHeader) {
            Log::error('No HMAC header in request');
            return false;
        }

        $data = $request->getContent();
        $calculatedHmac = base64_encode(hash_hmac('sha256', $data, env('SHOPIFY_API_SECRET'), true));
        if (!is_string($hmacHeader) || !is_string($calculatedHmac)) {
            Log::error('HMAC or calculated HMAC is not a string');
            return false;
        }

        return hash_equals($hmacHeader, $calculatedHmac);
    }

    public function createWebhook(Request $request)
    {
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if ($shop) {
        $client = $this->createClient($session);

        
        $webhookTopicUrlMap = [
            'orders/create' => route('order-create', ['shopId' => $shop->id]),
            'orders/delete' => route('order-delete', ['shopId' => $shop->id]),
            'orders/fulfilled' => route('order-fulfilled', ['shopId' => $shop->id]),
        ];

        foreach ($webhookTopicUrlMap as $topic => $webhookUrl) {
            try {
                $webhook = Webhook::where('shop_id', $shop->id)
                    ->where('webhook_name', $topic)
                    ->first();

                if (!$webhook) {
                    $webhookPayload = [
                        'webhook' => [
                            'topic' => $topic,
                            'address' => $webhookUrl,
                            'format' => 'json'
                        ]
                    ];

                    $client->post('webhooks.json', [
                        'json' => $webhookPayload
                    ]);

                   
                    Webhook::create([
                        'shop_id' => $shop->id,
                        'url_name' => $webhookUrl,
                        'webhook_name' => $topic,
                    ]);
                }
                else{
                    return response()->json([
                        'message' => 'webhook already created'
                    ], 200);
                }
            } catch (\Throwable $th) {
                continue;
            }
        }
    }
}  

        public function orderCreate(Request $request, $domain)
        {
            
            $topic = $request->header(HttpHeaders::X_SHOPIFY_TOPIC, '');
            try {
                if (!$this->verifyShopifyWebhook($request)) {
                    throw new InvalidWebhookException('Invalid webhook signature');
                } else {
                    // -------------------- Custom code --------------------
    
                    try {
    
                        $orderData = $request->all();
                        Log::info('createOrder ' .  json_encode($orderData));
                        $order_id = $orderData['id'];
    
                        // Log::info('New order created by ' .  json_encode($orderData));
    
                        if ($domain) {
                            $shop = Session::find($domain);
                            if (!$shop) {
                                return response()->json(['error' => 'Session not found'], 404);
                            }
                                                   
                        $accessDetail = $shop->accessDetail;
                           
                            if ($shop) {

                                                             
                                    // $data = [
                                    //     'access_id' => $accessDetail->id,
                                    //     'order_id' => $order_id,
                                    //     'order_data' => $orderData,
                                    //     'line_items'=> $orderData['line_items'],
                                    //     'status'=>0
                                    // ];
                                   Log::info('Order created');
                                   // $webhook_order = Order::create($data);
                               
    
                                    return response()->json([
                                        'message' => 'Order created successfully'
                                    ], 200);
                               
                            } else {
                                Log::info('Shop is not found');
                                return response()->json([
                                    'message' => 'Shopify header missing'
                                ], 200);
                            }
                        } else {
                            Log::info('Shopify header missing');
                            return response()->json([
                                'message' => 'Shopify header missing'
                            ], 200);
                        }
                        return response()->json(['message' => "successfully created an order"], 200);
                    } catch (\Throwable $th) {
                        Log::error("Got an exception when handling webhook: {$th->getMessage()}");
                        return response()->json(['message' => "Got an exception when handling webhook"], 500);
                    }
                    // -------------------- End Custom code --------------------
                }
            } catch (InvalidWebhookException $e) {
                return response()->json(['message' => "Got invalid webhook request for topic '$topic' "], 401);
            } catch (\Exception $e) {
                return response()->json(['message' => "Got an exception when handling '$topic' webhook"], 500);
            }
        }
        public function orderDelete(Request $request, $domain)
        {
            $topic = $request->header(HttpHeaders::X_SHOPIFY_TOPIC, '');
            try {
                if (!$this->verifyShopifyWebhook($request)) {
                    throw new InvalidWebhookException('Invalid webhook signature');
                } else {
                    // -------------------- Custom code --------------------
    
                    try {
    
                        $orderData = $request->all();
                        // Log::info('Order Delete ' .  json_encode($orderData));
                        $orderId = $orderData['id'];
                        // Log::info('Order Id ' .  $orderId);
                        // $domain = $request->header('X-Shopify-Shop-Domain');
                        Log::info('Order deletion requested by ' . $domain);
    
                        // Check the Shopify header
                        if ($domain) {
                            // $shop = Session::where('shop', $domain)->first();
                            $shop = Session::find($domain);
                          
                            
                            if ($shop) {
                                try {
                                    $accessDetail = $shop->accessDetail;
                                    $deleted = Order::where('order_id', $orderId)
                                    ->where('access_id', $accessDetail->id)
                                    ->delete();
    
                                    if ($deleted) {
                                        return response()->json([
                                            'message' => 'Order deleted successfully'
                                        ], 200);
                                    } else {
                                        return response()->json([
                                            'message' => 'Failed to delete order'
                                        ], 200);
                                    }
                                } catch (\Exception $e) {
                                    // Log the error
                                    Log::error('Failed to delete order: ' . $e->getMessage());
    
                                    return response()->json([
                                        'message' => 'Failed to delete order'
                                    ], 200);
                                }
                            } else {
                                Log::info('Shop not found');
    
                                return response()->json([
                                    'message' => 'Shop not found'
                                ], 200);
                            }
                        } else {
                            Log::info('Shopify header missing');
    
                            return response()->json([
                                'message' => 'Shopify header missing'
                            ], 200);
                        }
                    } catch (\Throwable $th) {
                        Log::error("Got an exception when handling webhook: {$th->getMessage()}");
                        return response()->json(['message' => "Got an exception when handling webhook"], 500);
                    }
                    // -------------------- End Custom code --------------------
                }
            } catch (InvalidWebhookException $e) {
                return response()->json(['message' => "Got invalid webhook request for topic '$topic' "], 401);
            } catch (\Exception $e) {
                return response()->json(['message' => "Got an exception when handling '$topic' webhook"], 500);
            }
        }

        public function clearWebhooks(Request $request, $id)
    {

        try {


            $session = Session::find($id);
            if ($session) {
                $accessToken = $session->access_token;
                $shop = $session->shop;

                $baseUri = "https://{$shop}/admin/api/2023-10/";

                $client = new Client([
                    'base_uri' => $baseUri,
                    'headers' => [
                        'Content-Type' => 'application/json',
                        'X-Shopify-Access-Token' => $accessToken,
                    ],
                ]);
                $response = $client->get('webhooks.json');
                $result = json_decode($response->getBody(), true);

                foreach ($result['webhooks'] as $key => $webhook) {
                    $webhookId = $webhook['id'];
                    $client->delete("webhooks/{$webhookId}.json");
                }

                $response = $client->get('webhooks.json');
                $remainingWebhooks = json_decode($response->getBody(), true)['webhooks'];
                return $remainingWebhooks;
            }
        } catch (\Exception $e) {
            Log::error("Got an exception when handling");
            return response()->json(['message' => "Got an exception when handling webhook"], 500);
        }
    }
    public function handleWebhook(Request $request)
    {
        // Handle incoming webhook data here
        $payload = $request->all(); // Get all data sent in the request
        
        // Process the data as per your application's requirements

        // Optionally, you can return a response to the third party indicating success or failure
        return response()->json(['message' => 'Webhook received successfully'], 200);
    }
    public function  orderFulfilled(Request $request, $domain)
    {
        $topic = $request->header(HttpHeaders::X_SHOPIFY_TOPIC, '');
        try {
            if (!$this->verifyShopifyWebhook($request)) {
                throw new InvalidWebhookException('Invalid webhook signature');
            } else {
                // --------------------Custom code --------------------

                try {
                    $orderData = $request->all();
                    // Log::info('orderFulfilled ' .  json_encode($orderData));
                    $orderId = $orderData['id'];
                    // $domain = $request->header('X-Shopify-Shop-Domain');
    
                    if ($domain) {
                        // $shop = Session::where('shop', $domain)->first();
                        $shop = Session::find($domain);
    
                        if ($shop) {
                            try {
                                $order = Order::where('order_id', $orderId)->first();
    
                                if ($order) {
                                    $allFulfilled = true; 
    
                                    foreach ($orderData['line_items'] as $key => $lineItem) {
                                      
                                        if ($lineItem['fulfillable_quantity'] != 0 && $lineItem['fulfillment_status'] != 'fulfilled') {
                                            $allFulfilled = false; 
                                        }
                                    }
    
                                    if ($allFulfilled) {
                                        $order->status = 1;
                                    }
                                    
                                    $order->save();
                              
                                    return response()->json([
                                        'message' => 'Order fulfilled successfully'
                                    ], 200);
                                } else {
                                    return response()->json([
                                        'message' => 'Failed to fulfill order'
                                    ], 200);
                                }
                            } catch (\Exception $e) {
                               
                                Log::error('Failed to fulfill order: ' . $e->getMessage());

                                return response()->json([
                                    'message' => 'Failed to fulfill order'
                                ], 500);
                            }
                        } else {
                            Log::info('Shop not found');

                            return response()->json([
                                'message' => 'Shop not found'
                            ], 200);
                        }
                    } else {
                        Log::info('Shopify header missing');

                        return response()->json([
                            'message' => 'Shopify header missing'
                        ], 200);
                    }
                } catch (\Throwable $th) {
                    Log::error("Got an exception when handling webhook: {$th->getMessage()}");
                    return response()->json(['message' => "Got an exception when handling webhook"], 500);
                }
                // -------------------- End Custom code --------------------
            }
        } catch (InvalidWebhookException $e) {
            return response()->json(['message' => "Got invalid webhook request for topic '$topic' "], 401);
        } catch (\Exception $e) {
            return response()->json(['message' => "Got an exception when handling '$topic' webhook"], 500);
        }
    }
}
