import React, { useState, useCallback,useEffect, useRef } from 'react';
import {
  Page,
  Layout,
  LegacyCard,
  Tabs,
  Form,
  FormLayout,
  Button,
  Toast,
  Frame ,EmptyState, Link,
  TextField,
  IndexTable,
  IndexFilters,
  useSetIndexFiltersMode,
  useIndexResourceState,
  Text,
  ChoiceList,
  RangeSlider,
  Badge,
  IndexFiltersMode,
  useBreakpoints,DatePicker, Pagination,
  Spinner,Image, Modal,Select,Checkbox 
} from "@shopify/polaris";

import { TitleBar } from "@shopify/app-bridge-react";
import { useTranslation } from "react-i18next";
import { useAppQuery, useAuthenticatedFetch } from "../hooks";
import './style.css';
import { skyexlogo } from "../assets";
import { ChevronDownMinor, ChevronUpMinor } from '@shopify/polaris-icons';


export default function Orders() {

    const { t } = useTranslation();
    const fetch = useAuthenticatedFetch();
    const [orderData, setOrders] = useState([]);
    const [pushedOrder, setPushedOrders] = useState([]);
    const [removedOrders, setRemovedOrders] = useState([]);
    const [pendingOrders, setPendingOrders] = useState([]);
    const [currentPage, setCurrentPage] = useState(1);
    const [pushedCurrentPage, setPushedCurrentPage] = useState(1);
    const [currentPageRemove, setCurrentPageRemove] = useState(1);
    const itemsPerPage = 10;
    const [paginatedOrders, setPaginatedOrders] = useState([]);
    const [pushedPaginatedOrders, setPushedPaginatedOrders] = useState([]);
    const [paginatedOrderRemove, setPaginatedOrderRemove] = useState([]);
    const [isLoading, setIsLoading] = useState(true);
    const [loadingRows, setLoadingRows] = useState({});
    const [loadingRowsDownload, setLoadingRowsDownload] = useState({});
    const [reloadLoader, setReloadLoader] = useState({});
    const [reloadLoaderRemove, setReloadLoaderRemove] = useState({});
    

    const [returnedOrders, setReturnedOrders] = useState([]);
    const [returnedOrdersProducts, setReturnedOrdersProducts] = useState([]);
    const [paginatedOrderReturn, setPaginatedOrderReturn] = useState([]);
    const [currentPageReturn, setCurrentPageReturn] = useState(1);
    const [isDeclineModalOpen, setIsDeclineModalOpen] = useState(false);
    const [declineReason, setDeclineReason] = useState('');
    const [lineItemId, setlineItemId] = useState('');
    const [currentLineItemId, setCurrentLineItemId] = useState('');
    const [currentOrderId, setCurrentOrderId] = useState('');
    const [currentReturnId, setCurrentReturnId] = useState('');
    const [currentOrderNumber, setCurrentOrderNumber] = useState('');
    const [declineReasonError, setDeclineReasonError] = useState('');
    const [returnStatuses, setReturnStatuses] = useState({});
    const [isBulkApprove, setIsBulkApprove] = useState(false);
    const [isBulkDownloadReturn, setIsBulkDownloadReturn] = useState(false);
    const [isBulkDecline, setIsBulkDecline] = useState(false);
    const [disabledCheckboxes, setDisabledCheckboxes] = useState({});

    const [isBulkDownloading, setIsBulkDownloading] = useState(false);
    const [isBulkAction, setIsBulkAction] = useState(false);
    const [isBulkRemove, setIsBulkRemove] = useState(false);
    const [isBulkReload, setIsBulkReload] = useState(false);
   
    const [companyDetails, setCompanyDetails] = useState([]);
    const [isModalVisible, setIsModalVisible] = useState(false);
    const [selectedCompanyIds, setSelectedCompanyIds] = useState({});
    const [active, setActive] = useState(false);
    const [selectedModal, setSelectedModal] = useState('');
    
    const [activeRemove, setActiveRemove] = useState(false);
    const [selectedModalRemove, setSelectedModalRemove] = useState('');
        const activeDownloads = useRef({});
    const emptyToastProps = { content: null };
	const [toastProps, setToastProps] = useState(emptyToastProps); 
  const toastMarkup = toastProps.content && (
		<Toast {...toastProps} onDismiss={() => setToastProps(emptyToastProps)} />
	);

  const handleModalChange = useCallback(() => setActive(!active), [active]);
  const handleSelectChange = useCallback((value) => setSelectedModal(value), []);

  const handleSave = () => {
   
    const selectedOrderIds = allResourcesSelected ? orderData.map(order => order.id) : selectedResources;
  
    const updatedSelectedCompanyIds = { ...selectedCompanyIds };
  
    selectedOrderIds.forEach((orderId) => {
      updatedSelectedCompanyIds[orderId] = selectedModal;
    });
  
    setSelectedCompanyIds(updatedSelectedCompanyIds);
  
    handleModalChange();
  };

  const handleModalChangeRemove = useCallback(() => setActiveRemove(!activeRemove), [activeRemove]);
  const handleSelectChangeRemove = useCallback((value) => setSelectedModalRemove(value), []);


  // const [selectedAPI, setSelectedAPI] = useState(null);
  // const { data } = useAppQuery({
  //   url: "/api/checkChangeApiStatusdata",
  //   reactQueryOptions: {
  //     onSuccess: (response) => {
  //       setSelectedAPI(response.buttonChangeApiStatus);
  //       //console.log(response.apiStatus);
  //     },
  //   },
  // });

  const [selectedAPI, setSelectedAPI] = useState(null);

useEffect(() => {
  const fetchData = async () => {
    try {
      const response = await fetch("/api/checkChangeApiStatusdata");
      const result = await response.json();
      setSelectedAPI(result.buttonChangeApiStatus);
      //console.log(result.apiStatus);
    } catch (error) {
      console.error("Error fetching data:", error);
    }
  };

  fetchData();
}, []);



  const handleSaveRemove = () => {
    const selectedOrderIds = allResourcesSelectedRemove ? removedOrders.map(order => order.id) : selectedResourcesRemove;

     const updatedSelectedCompanyIds = { ...selectedCompanyIds };
  
    selectedOrderIds.forEach((orderId) => {
      updatedSelectedCompanyIds[orderId] = selectedModalRemove;
    });
  
    setSelectedCompanyIds(updatedSelectedCompanyIds);
  
    handleModalChangeRemove();
  };

  const promotedBulkActions = [
    ...(selectedAPI === 1 && companyDetails.length > 1 ? [
      {
        content:'Change Location',
        onAction: handleModalChange,
      }
    ] : []),
    {
      content: isBulkAction ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Push To SkyEx',
      onAction: () => {
        if (selectedAPI === 1) {
          handleBulkAction();
        } else {
          handleBulkActionSkyEx();
        }
      },

    },
    {
      content: isBulkRemove ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Push to RemoveList',
      onAction: () => {
        if (selectedAPI === 1) {
          handleBulkActionRemove();
        } else {
          handleBulkActionRemoveSkyEx();
        }
      },
    },
    ...(selectedAPI === 1
      ? [
          {
      content: isBulkReload ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Reload Data',
      onAction: () =>handleBulkActionReload() ,
      
    },
  ]
  : []),
  ];
  const promotedBulkActionsPush = [
    {
      content: isBulkDownloading ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Bulk Download',
      
      onAction: () => {
        if (selectedAPI === 1) {
          handleBulkActionDownload();
        } else {
          handleBulkActionDownloadSkyEx();
        }
      },
    },
   
  ];

  const promotedBulkActionsRemove = [
    ...(selectedAPI === 1 && companyDetails.length > 1 ? [
      {
        content:'Change Location',
        onAction: handleModalChangeRemove,
      }
    ] : []),
    {
      content: isBulkAction ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Push To SkyEx',
      
      onAction: () =>handleBulkActionPush() ,
      onAction: () => {
        if (selectedAPI === 1) {
          handleBulkActionPush();
        } else {
          handleBulkActionPushSkyEx();
        }
      },
    },
    ...(selectedAPI === 1
      ? [
          {
            content: isBulkReload ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Reload Data',
            onAction: () => handleBulkActionReloadRemove(),
          },
        ]
      : []),
   
  ];
  const { data: dataCompany } = useAppQuery({
    url: "/api/getCompanies",
    reactQueryOptions: {
      onSuccess: (response) => {
        setIsLoading(false);
       
        const sortedCompanies = response.details.sort((a, b) => {
          if (a.is_default) return -1; 
          if (b.is_default) return 1;  
          return 0;
        });
        setCompanyDetails(sortedCompanies);
                
       
      },
    },
  });
  useEffect(() => {
   
      const initialSelectedCompanyIds = {};
      [...orderData, ...removedOrders].forEach(order => {
        initialSelectedCompanyIds[order.id] = companyDetails.find(company => company.is_default)?.id.toString() || '';
      });
      setSelectedCompanyIds(initialSelectedCompanyIds);
    
  }, [orderData, removedOrders, companyDetails]);

  const CompanyOptions = companyDetails.map(company => ({
    label: company.company_name, 
    value: company.id.toString(), 
}));

const handleCompanyChange = (orderId, selectedCompanyId) => {
  setSelectedCompanyIds(prevSelectedCompanyIds => ({
    ...prevSelectedCompanyIds,
    [orderId]: selectedCompanyId,
  }));
};


const promotedBulkActionsReturnProducts = [
    
  {
    content: isBulkApprove ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Bulk Approve',
    onAction: () =>handleBulkActionApproveReturn() ,

  }
  // ,
  // {
  //   content: isBulkDownloadReturn ? <Spinner accessibilityLabel="Downloading" size="small" /> : 'Download',
  //   onAction: () =>handleBulkActionDownloadReturn() ,

  // }
];


const handleBulkActionDownload = async () => {
  try {
      setLoadingRows({});
      setIsBulkDownloading(true);

      const selectedOrderIds = allResourcesSelectedPush ? pushedOrder.map(order => order.id) : selectedResourcesPush;

      const response = await fetch('/api/getDownloadLabelAll', {
          method: 'POST',
          headers: {
              'Content-Type': 'application/json',
          },
          body: JSON.stringify({
              orderIds: selectedOrderIds,
          }),
      });

      if (!response.ok) {
          throw new Error('Failed to fetch merged PDF.');
      }

      const blob = await response.blob();
      const url = window.URL.createObjectURL(blob);

      setToastProps({
          content: "Bulk download completed successfully",
      });

      
      const a = document.createElement('a');
      a.style.display = 'none';
      a.href = url;
      a.download = 'merged_labels.pdf'; 
      document.body.appendChild(a);
      a.click();
      window.URL.revokeObjectURL(url);
      document.body.removeChild(a);

      handleSelectionChangePush([]);

  } catch (error) {
      console.error('Error downloading labels:', error);
      setToastProps({
          content: "Error downloading labels",
          error: true,
      });
  } finally {
      setLoadingRows({});
      setIsBulkDownloading(false);
  }
};


// const handleBulkActionDownloadSkyEx = async () => {
//   try {
//       setLoadingRows({});
//       setIsBulkDownloading(true);

//       const selectedOrderIds = allResourcesSelectedPush ? pushedOrder.map(order => order.id) : selectedResourcesPush;

//       const response = await fetch('/api/getDownloadLabelAllSkyEx', {
//           method: 'POST',
//           headers: {
//               'Content-Type': 'application/json',
//           },
//           body: JSON.stringify({
//               orderIds: selectedOrderIds,
//           }),
//       });

//       if (!response.ok) {
//           throw new Error('Failed to fetch merged PDF.');
//       }

//       const blob = await response.blob();
//       const url = window.URL.createObjectURL(blob);

//       setToastProps({
//           content: "Bulk download completed successfully",
//       });

      
//       const a = document.createElement('a');
//       a.style.display = 'none';
//       a.href = url;
//       a.download = 'merged_labels.pdf'; 
//       document.body.appendChild(a);
//       a.click();
//       window.URL.revokeObjectURL(url);
//       document.body.removeChild(a);

//       handleSelectionChangePush([]);

//   } catch (error) {
//       console.error('Error downloading labels:', error);
//       setToastProps({
//           content: "Error downloading labels",
//           error: true,
//       });
//   } finally {
//       setLoadingRows({});
//       setIsBulkDownloading(false);
//   }
// };



const handleBulkActionDownloadSkyEx = async () => {
  try {
    setLoadingRows({});
    setIsBulkDownloading(true);

    const selectedOrderIds = allResourcesSelectedPush ? pushedOrder.map(order => order.id) : selectedResourcesPush;

    const response = await fetch('/api/getDownloadLabelAllSkyEx', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({
        orderIds: selectedOrderIds,
      }),
    });

    if (!response.ok) {
      throw new Error('Failed to fetch merged PDF.');
    }

    const blob = await response.blob();
    const filename = response.headers.get('Content-Disposition')?.split('filename=')[1] || 'merged_labels.pdf';
    const url = window.URL.createObjectURL(blob);

    const a = document.createElement('a');
    a.style.display = 'none';
    a.href = url;
    a.download = filename;
    document.body.appendChild(a);
    a.click();
    window.URL.revokeObjectURL(url);
    document.body.removeChild(a);

    setToastProps({
      content: "Bulk download completed successfully",
    });

    handleSelectionChangePush([]);

  } catch (error) {
    console.error('Error downloading labels:', error);
    setToastProps({
      content: "Error downloading labels",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkDownloading(false);
  }
};

const handleBulkAction = async () => {

  try {
    setLoadingRows({});
    setIsBulkAction(true);

    const selectedOrderIds = allResourcesSelected ? orderData.map(order => order.id) : selectedResources;

    const responses = await Promise.all(selectedOrderIds.map((orderId) => {
      return fetch(`/api/getAWBNumber`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({ 
          orderId: orderId,
          selectedCompanyId: selectedCompanyIds[orderId] 
        }),
      });
    }));

    const responseData = await Promise.all(responses.map(async (response, index) => {
      try {
        if (response.ok) {
          const data = await response.json();
          return { success: true, ...data, orderId: selectedOrderIds[index] };
        } else {
          const errorData = await response.json();
          return { success: false, error: errorData.error, orderId: selectedOrderIds[index] };
        }
      } catch (error) {
        console.error('Error parsing response:', error);
        throw error;
      }
    }));

    const successfulOrders = responseData.filter(res => res.success);
    const failedOrders = responseData.filter(res => !res.success);

    const newOrders = successfulOrders.map(data => data.orders).flat();
    const newPushedOrders = successfulOrders.map(data => data.pushed_orders).flat();
    const newRemovedOrders = successfulOrders.map(data => data.removed_orders).flat();

    const errorMessages = failedOrders.map(data => data.error);

    const successfulOrderIds = successfulOrders.map(data => data.orderId);

    // Update orderData by removing successfully pushed orders
    setOrders(prevOrders => {
      const updatedOrders = prevOrders.filter(order => !successfulOrderIds.includes(order.id));
      return updatedOrders;
    });

    // Update pushedOrders by adding new pushed orders and removing duplicates
    // setPushedOrders(prevPushedOrders => {
    //   const combinedPushedOrders = [...prevPushedOrders, ...newPushedOrders];
    //   return Array.from(new Set(combinedPushedOrders.map(order => order.id)))
    //               .map(id => combinedPushedOrders.find(order => order.id === id));
    // });
    setPushedOrders(prevPushedOrders => {
      const combinedPushedOrders = [...newPushedOrders, ...prevPushedOrders];
      
      const uniqueOrders = Array.from(new Set(combinedPushedOrders.map(order => order.id)))
                                .map(id => combinedPushedOrders.find(order => order.id === id))
                                .sort((a, b) => {
                                  
                                  if (a.order_no > b.order_no) return -1;
                                  if (a.order_no < b.order_no) return 1;
                                 
                                  return new Date(b.created_at) - new Date(a.created_at);
                                });

      return uniqueOrders;
    });
    
    if (errorMessages.length > 0) {
      setToastProps({
        content: `Errors: ${errorMessages.join(', ')}`,
        error: true,
      });
    } else {
      setToastProps({
        content: "Pushed to SkyEx Successfully",
      });
    }

  } catch (error) {
    console.error('Error pushing orders to SkyEx:', error);
    setToastProps({
      content: "Error pushing orders to SkyEx",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkAction(false);
  }
};

const handleBulkActionSkyEx = async () => {

  try {
    setLoadingRows({});
    setIsBulkAction(true);

    const selectedOrderIds = allResourcesSelected ? orderData.map(order => order.id) : selectedResources;

    const responses = await Promise.all(selectedOrderIds.map((orderId) => {
      return fetch(`/api/skyExGetAWBNumber`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({ 
          orderId: orderId,
        }),
      });
    }));

    const responseData = await Promise.all(responses.map(async (response, index) => {
      try {
        if (response.ok) {
          const data = await response.json();
          return { success: true, ...data, orderId: selectedOrderIds[index] };
        } else {
          const errorData = await response.json();
          return { success: false, error: errorData.error, orderId: selectedOrderIds[index] };
        }
      } catch (error) {
        console.error('Error parsing response:', error);
        throw error;
      }
    }));

    const successfulOrders = responseData.filter(res => res.success);
    const failedOrders = responseData.filter(res => !res.success);

    const newOrders = successfulOrders.map(data => data.orders).flat();
    const newPushedOrders = successfulOrders.map(data => data.pushed_orders).flat();
    const newRemovedOrders = successfulOrders.map(data => data.removed_orders).flat();

    const errorMessages = failedOrders.map(data => data.error);

    const successfulOrderIds = successfulOrders.map(data => data.orderId);

    // Update orderData by removing successfully pushed orders
    setOrders(prevOrders => {
      const updatedOrders = prevOrders.filter(order => !successfulOrderIds.includes(order.id));
      return updatedOrders;
    });

    // Update pushedOrders by adding new pushed orders and removing duplicates
    // setPushedOrders(prevPushedOrders => {
    //   const combinedPushedOrders = [...prevPushedOrders, ...newPushedOrders];
    //   return Array.from(new Set(combinedPushedOrders.map(order => order.id)))
    //               .map(id => combinedPushedOrders.find(order => order.id === id));
    // });
    setPushedOrders(prevPushedOrders => {
      const combinedPushedOrders = [...newPushedOrders, ...prevPushedOrders];
      
      const uniqueOrders = Array.from(new Set(combinedPushedOrders.map(order => order.id)))
                                .map(id => combinedPushedOrders.find(order => order.id === id))
                                .sort((a, b) => {
                                  
                                  if (a.order_no > b.order_no) return -1;
                                  if (a.order_no < b.order_no) return 1;
                                 
                                  return new Date(b.created_at) - new Date(a.created_at);
                                });

      return uniqueOrders;
    });
    
    if (errorMessages.length > 0) {
      setToastProps({
        content: `Errors: ${errorMessages.join(', ')}`,
        error: true,
      });
    } else {
      setToastProps({
        content: "Pushed to SkyEx Successfully",
      });
    }

  } catch (error) {
    console.error('Error pushing orders to SkyEx:', error);
    setToastProps({
      content: "Error pushing orders to SkyEx",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkAction(false);
  }
};

const handleBulkActionReload = async () => {
  try {
    setLoadingRows({});
    setIsBulkReload(true);

    const selectedOrderIds = allResourcesSelected ? orderData.map(order => order.id) : selectedResources;

    const response = await fetch('/api/getReloadDataAll', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({
        orderIds: selectedOrderIds,
      }),
    });

    if (!response.ok) {
      throw new Error('Failed to reload.');
    }

    const result = await response.json();

    if (result && result.updatedOrders) {
      console.log(result.updatedOrders);
      setOrders(prevOrderData => {
        return prevOrderData.map(order => {
          const updatedOrder = result.updatedOrders.find(updated => updated.order_id === order.order_id);
          return updatedOrder ? { ...order, ...updatedOrder } : order;
        });
      });
      setToastProps({
        content: "Reload successful.",
        error: false,
      });
    }
  } catch (error) {
    console.error('Error reloading:', error);
    setToastProps({
      content: "Failed to reload.",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkReload(false);
  }
};

const handleBulkActionReloadRemove = async () => {
  try {
    setLoadingRows({});
    setIsBulkReload(true);

    const selectedOrderIds = allResourcesSelectedRemove ? removedOrders.map(order => order.id) : selectedResourcesRemove;

    const response = await fetch('/api/getReloadDataAll', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({
        orderIds: selectedOrderIds,
      }),
    });

    if (!response.ok) {
      throw new Error('Failed to reload.');
    }

    const result = await response.json();

    if (result && result.updatedOrders) {
      console.log(result.updatedOrders);
      setRemovedOrders(prevOrderData => {
        return prevOrderData.map(order => {
          const updatedOrder = result.updatedOrders.find(updated => updated.order_id === order.order_id);
          return updatedOrder ? { ...order, ...updatedOrder } : order;
        });
      });
      setToastProps({
        content: "Reload successful.",
        error: false,
      });
    }
  } catch (error) {
    console.error('Error reloading:', error);
    setToastProps({
      content: "Failed to reload.",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkReload(false);
  }
};
const handleBulkActionPush = async () => {
  try {
    setLoadingRows({});
    setIsBulkAction(true);

    const selectedOrderIds = allResourcesSelectedRemove ? removedOrders.map(order => order.id) : selectedResourcesRemove;

    const responses = await Promise.all(selectedOrderIds.map(async (orderId) => {
      try {
        const response = await fetch(`/api/getAWBNumber`, {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({
            orderId: orderId,
          }),
        });

        if (!response.ok) {
          const errorData = await response.json();
          return { success: false, error: errorData.error, orderId: orderId };
        }

        const data = await response.json();
        return { success: true, ...data, orderId: orderId };
      } catch (error) {
        console.error('Error in fetch:', error);
        return { success: false, error: error.message, orderId: orderId };
      }
    }));

    const successfulOrders = responses.filter(res => res.success);
    const failedOrders = responses.filter(res => !res.success);

    const newPushedOrders = successfulOrders.map(data => data.pushed_orders).flat();
    const successfulOrderIds = successfulOrders.map(data => data.orderId);

    const errorMessages = failedOrders.map(data => data.error);

    // Update pushedOrders by adding new pushed orders and removing duplicates
    // setPushedOrders(prevPushedOrders => {
    //   const combinedPushedOrders = [...prevPushedOrders, ...newPushedOrders];
    //   return Array.from(new Set(combinedPushedOrders.map(order => order.id)))
    //               .map(id => combinedPushedOrders.find(order => order.id === id));
    // });
    setPushedOrders(prevPushedOrders => {
      
      const combinedPushedOrders = [...newPushedOrders, ...prevPushedOrders];
      
    
      const uniqueOrders = Array.from(new Set(combinedPushedOrders.map(order => order.id)))
        .map(id => combinedPushedOrders.find(order => order.id === id));
    
     
      const sortedOrders = uniqueOrders.sort((a, b) => {
       
        if (a.order_no > b.order_no) return -1;
        if (a.order_no < b.order_no) return 1;
        
        return new Date(b.created_at) - new Date(a.created_at);
      });
    
      return sortedOrders;
    });

    // Update removedOrders by removing successfully pushed orders
    setRemovedOrders(prevRemovedOrders => {
      const updatedRemovedOrders = prevRemovedOrders.filter(order => !successfulOrderIds.includes(order.id));
      return updatedRemovedOrders;
    });

    if (errorMessages.length > 0) {
      setToastProps({
        content: `Errors: ${errorMessages.join(', ')}`,
        error: true,
      });
    } else {
      setToastProps({
        content: "Pushed to SkyEx Successfully",
      });
    }

  } catch (error) {
    console.error('Error pushing orders to SkyEx:', error);
    setToastProps({
      content: "Error pushing orders to SkyEx",
      error: true,
    });
  } finally {
    setLoadingRows(false);
    setIsBulkAction(false);
  }
};

const handleBulkActionPushSkyEx = async () => {
  try {
    setLoadingRows({});
    setIsBulkAction(true);

    const selectedOrderIds = allResourcesSelectedRemove ? removedOrders.map(order => order.id) : selectedResourcesRemove;

    const responses = await Promise.all(selectedOrderIds.map(async (orderId) => {
      try {
        const response = await fetch(`/api/skyExGetAWBNumber`, {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({
            orderId: orderId,
          }),
        });

        if (!response.ok) {
          const errorData = await response.json();
          return { success: false, error: errorData.error, orderId: orderId };
        }

        const data = await response.json();
        return { success: true, ...data, orderId: orderId };
      } catch (error) {
        console.error('Error in fetch:', error);
        return { success: false, error: error.message, orderId: orderId };
      }
    }));

    const successfulOrders = responses.filter(res => res.success);
    const failedOrders = responses.filter(res => !res.success);

    const newPushedOrders = successfulOrders.map(data => data.pushed_orders).flat();
    const successfulOrderIds = successfulOrders.map(data => data.orderId);

    const errorMessages = failedOrders.map(data => data.error);

    // Update pushedOrders by adding new pushed orders and removing duplicates
    // setPushedOrders(prevPushedOrders => {
    //   const combinedPushedOrders = [...prevPushedOrders, ...newPushedOrders];
    //   return Array.from(new Set(combinedPushedOrders.map(order => order.id)))
    //               .map(id => combinedPushedOrders.find(order => order.id === id));
    // });
    setPushedOrders(prevPushedOrders => {
      
      const combinedPushedOrders = [...newPushedOrders, ...prevPushedOrders];
      
    
      const uniqueOrders = Array.from(new Set(combinedPushedOrders.map(order => order.id)))
        .map(id => combinedPushedOrders.find(order => order.id === id));
    
     
      const sortedOrders = uniqueOrders.sort((a, b) => {
       
        if (a.order_no > b.order_no) return -1;
        if (a.order_no < b.order_no) return 1;
        
        return new Date(b.created_at) - new Date(a.created_at);
      });
    
      return sortedOrders;
    });

    // Update removedOrders by removing successfully pushed orders
    setRemovedOrders(prevRemovedOrders => {
      const updatedRemovedOrders = prevRemovedOrders.filter(order => !successfulOrderIds.includes(order.id));
      return updatedRemovedOrders;
    });

    if (errorMessages.length > 0) {
      setToastProps({
        content: `Errors: ${errorMessages.join(', ')}`,
        error: true,
      });
    } else {
      setToastProps({
        content: "Pushed to SkyEx Successfully",
      });
    }

  } catch (error) {
    console.error('Error pushing orders to SkyEx:', error);
    setToastProps({
      content: "Error pushing orders to SkyEx",
      error: true,
    });
  } finally {
    setLoadingRows(false);
    setIsBulkAction(false);
  }
};

  const handleBulkActionRemove = async () => {
    try {
        setLoadingRows({});
        setIsBulkRemove(true);

        const selectedOrderIds = allResourcesSelected ? orderData.map(order => order.id) : selectedResources;

        const responses = await Promise.all(selectedOrderIds.map(orderId => {
            return fetch('/api/getRemoveAll', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    orderIds: [orderId], 
                }),
            });
        }));

        const responseData = await Promise.all(responses.map(async (response) => {
            try {
                if (response.ok) {
                    const data = await response.json();
                    return { success: true, data };
                } else {
                    const errorData = await response.json();
                    return { success: false, error: errorData.error };
                }
            } catch (error) {
                console.error('Error parsing response:', error);
                throw error;
            }
        }));

        
        const successfulResponses = responseData.filter(res => res.success).map(res => res.data);
       
        const errorResponses = responseData.filter(res => !res.success).map(res => res.error);

        
        setOrders(prevOrders => {
           console.log(prevOrders);
            const updatedOrders = prevOrders.filter(order => !selectedOrderIds.includes(order.id));
            return updatedOrders;
        });
       
  //     setRemovedOrders(prevRemovedOrders => {
  //     const updatedRemovedOrders = prevRemovedOrders.concat(orderData.filter(order => selectedOrderIds.includes(order.id)));
  //     return updatedRemovedOrders;
  // });
      
  setRemovedOrders(prevRemovedOrders => {
      const newRemovedOrders = orderData.filter(order => selectedOrderIds.includes(order.id));
    
    const updatedRemovedOrders = [...newRemovedOrders, ...prevRemovedOrders];
    
    const sortedRemovedOrders = updatedRemovedOrders.sort((a, b) => {
     
      if (a.order_no > b.order_no) return -1;
      if (a.order_no < b.order_no) return 1;
    
      return new Date(b.created_at) - new Date(a.created_at);
    });
  
    return sortedRemovedOrders;
  });
       
        if (errorResponses.length > 0) {
            setToastProps({
                content: "Some orders failed to remove",
                error: true,
            });
        } else {
            setToastProps({
                content: "Orders pushed to remove list successfully"
            });
        }

    } catch (error) {
        console.error('Error removing orders:', error);
        setToastProps({
            content: "Error removing orders",
            error: true,
        });
    } finally {
        setLoadingRows({});
        setIsBulkRemove(false);
    }
};


const handleBulkActionRemoveSkyEx = async () => {
  try {
      setLoadingRows({});
      setIsBulkRemove(true);

      const selectedOrderIds = allResourcesSelected ? orderData.map(order => order.id) : selectedResources;

      const responses = await Promise.all(selectedOrderIds.map(orderId => {
          return fetch('/api/getRemoveAllSkyEx', {
              method: 'POST',
              headers: {
                  'Content-Type': 'application/json',
              },
              body: JSON.stringify({
                  orderIds: [orderId], 
              }),
          });
      }));

      const responseData = await Promise.all(responses.map(async (response) => {
          try {
              if (response.ok) {
                  const data = await response.json();
                  return { success: true, data };
              } else {
                  const errorData = await response.json();
                  return { success: false, error: errorData.error };
              }
          } catch (error) {
              console.error('Error parsing response:', error);
              throw error;
          }
      }));

      
      const successfulResponses = responseData.filter(res => res.success).map(res => res.data);
     
      const errorResponses = responseData.filter(res => !res.success).map(res => res.error);

      
      setOrders(prevOrders => {
         console.log(prevOrders);
          const updatedOrders = prevOrders.filter(order => !selectedOrderIds.includes(order.id));
          return updatedOrders;
      });
     
//     setRemovedOrders(prevRemovedOrders => {
//     const updatedRemovedOrders = prevRemovedOrders.concat(orderData.filter(order => selectedOrderIds.includes(order.id)));
//     return updatedRemovedOrders;
// });
    
setRemovedOrders(prevRemovedOrders => {
    const newRemovedOrders = orderData.filter(order => selectedOrderIds.includes(order.id));
  
  const updatedRemovedOrders = [...newRemovedOrders, ...prevRemovedOrders];
  
  const sortedRemovedOrders = updatedRemovedOrders.sort((a, b) => {
   
    if (a.order_no > b.order_no) return -1;
    if (a.order_no < b.order_no) return 1;
  
    return new Date(b.created_at) - new Date(a.created_at);
  });

  return sortedRemovedOrders;
});
     
      if (errorResponses.length > 0) {
          setToastProps({
              content: "Some orders failed to remove",
              error: true,
          });
      } else {
          setToastProps({
              content: "Orders pushed to remove list successfully"
          });
      }

  } catch (error) {
      console.error('Error removing orders:', error);
      setToastProps({
          content: "Error removing orders",
          error: true,
      });
  } finally {
      setLoadingRows({});
      setIsBulkRemove(false);
  }
};

const handleBulkActionApproveReturn = async () => {
  try {
    setLoadingRows({});
    setIsBulkApprove(true);

    const selectedItemDetails = selectedReturnLineItems;
    console.log(selectedItemDetails);

    const returnApproveSubmit = async (returnId, orderNumber,lineItemId,orderId) => {
      const payload = {
        returnId: returnId,
        orderNumber: orderNumber,
        lineItemId:lineItemId,
        orderId:orderId
      };
      console.log(payload);
      
      console.log(lineItemId);
      try {
        const response = await fetch('/api/returnApproveSubmit', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(payload),
        });

        if (!response.ok) {
          throw new Error('Network response was not ok');
        }

        const data = await response.json();
        console.log(data);

        if (data.error) {
          console.log(data.error);
          setToastProps({
            content: data.error,
            error: true,
          });
        } else {
          setReturnStatuses(prev => ({ ...prev, [returnId]: 'Approved' }));
          setDisabledCheckboxes((prev) => ({ ...prev, [returnId]: true }));
          fetchOrderDetails();
          fetchOrderSkyexDetails();
          setToastProps({
            content: "Order return request approved",
          });
        }
      } catch (error) {
        console.error('Error:', error);
        setToastProps({
          content: error.message,
          error: true,
        });
      }
    };

    await Promise.all(selectedItemDetails.map(async (item) => {
      const [orderNumber, returnId, lineItemId,orderId] = item.split('-');
      console.log(`OrderId: ${orderId}, Return ID: ${returnId}`);
      
      await returnApproveSubmit(returnId, orderNumber,lineItemId,orderId);
    }));
    
  } catch (error) {
    console.error('Error handling bulk action:', error);
    setToastProps({
      content: "Error handling bulk action",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkApprove(false);
  }
};



const handleBulkActionDownloadReturn = async () => {
  try {
    setLoadingRows({});
    setIsBulkDownloading(true);

    const selectedItemDetails = selectedReturnLineItems;
    console.log(selectedItemDetails);

    const getDownloadLabelAllReturn = async (returnId,orderNumber,lineItemId,orderId) => {
      const payload = {
        returnId: returnId,
        orderNumber: orderNumber,
        lineItemId:lineItemId,
        orderId:orderId
      };

      
      console.log(payload);
      try {
        const response = await fetch('/api/getDownloadLabelAllReturn', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(payload),
        });

        if (!response.ok) {
          throw new Error('Network response was not ok');
        }

        const blob = await response.blob();
        const url = window.URL.createObjectURL(blob);
  
        setToastProps({
            content: "Bulk download completed successfully",
        });
  
        
        const a = document.createElement('a');
        a.style.display = 'none';
        a.href = url;
        a.download = 'merged_labels.pdf'; 
        document.body.appendChild(a);
        a.click();
        window.URL.revokeObjectURL(url);
        document.body.removeChild(a);
  
        handleSelectionChangeReturn([]);
      } catch (error) {
        console.error('Error:', error);
        setToastProps({
          content: error.message,
          error: true,
        });
      }
    };

    await Promise.all(selectedItemDetails.map(async (item) => {
      const [orderNumber, returnId, lineItemId,orderId] = item.split('-');
      //console.log(`Order Number: ${orderNumber}, Return ID: ${returnId}`);
      //console.log(orderId);
      await getDownloadLabelAllReturn(returnId, orderNumber,lineItemId,orderId);
    }));
    
  } catch (error) {
    console.error('Error handling bulk action:', error);
    setToastProps({
      content: "Error handling bulk action",
      error: true,
    });
  } finally {
    setLoadingRows({});
    setIsBulkDownloading(false);
  }
};


    const [itemStrings, setItemStrings] = useState([]);

    const indexTabs = itemStrings.map((item, index) => ({
      content: item,
      index,
      id: `${item}-${index}`,
      isLocked: index === 0,
    }));
    const [selected, setSelected] = useState(0);

    

    const fetchOrderDetails = async() =>{
      setIsLoading(true);
     try{
      const response = await fetch('/api/getOrders',{
        method:'GET',
        headers:{
          'Content-Type':'application/json'
        }
  
      });
      if(!response.ok)
      {
        throw new Error('Network response was not ok');
      }
      const data= await response.json();
      console.log(data.orders);
      setOrders(data.orders);
      setPushedOrders(data.pushed_orders);
      setRemovedOrders(data.removed_orders);
      setReturnedOrders(data.returned_orders);

      const statuses = {};
      const updatedDisabledCheckboxes = { ...disabledCheckboxes };
      
      data.returned_orders.forEach(order => {
        try {
          const details = JSON.parse(order.approve_decline_details);
      
          if (Array.isArray(details)) {
            details.forEach(returnDetail => {
              const returnId = returnDetail.id.split('/').pop();
              if (returnDetail.status === 'OPEN') {
                statuses[returnId] = 'Approved';
                updatedDisabledCheckboxes[returnId] = true;
              } else if (returnDetail.status === 'DECLINED') {
                statuses[returnId] = 'Declined';
                updatedDisabledCheckboxes[returnId] = true;
              } else {
                statuses[returnId] = returnDetail.status;
              }
            });
          } else {
            console.warn(`Expected an array but got ${typeof details}`);
          }
        } catch (error) {
          console.error(`Error parsing JSON for order ${order.id}:`, error);
        }
      });
      
      setReturnStatuses(statuses);
      setDisabledCheckboxes(updatedDisabledCheckboxes);

      setIsLoading(false);
      
     }
     catch (error){
      console.error('Error fetching data:', error);
     }
      
    };

    const fetchOrderSkyexDetails = async() =>{
      setIsLoading(true);
     try{
      const response = await fetch('/api/getSkyexOrders',{
        method:'GET',
        headers:{
          'Content-Type':'application/json'
        }
  
      });
      if(!response.ok)
      {
        throw new Error('Network response was not ok');
      }
      const data= await response.json();
      console.log(data.orders);
      setOrders(data.orders);
      setPushedOrders(data.pushed_orders);
      setRemovedOrders(data.removed_orders);
      setReturnedOrders(data.returned_orders);

      const statuses = {};
      const updatedDisabledCheckboxes = { ...disabledCheckboxes };
      
      data.returned_orders.forEach(order => {
        try {
          const details = JSON.parse(order.approve_decline_details);
      
          if (Array.isArray(details)) {
            details.forEach(returnDetail => {
              const returnId = returnDetail.id.split('/').pop();
              if (returnDetail.status === 'OPEN') {
                statuses[returnId] = 'Approved';
                updatedDisabledCheckboxes[returnId] = true;
              } else if (returnDetail.status === 'DECLINED') {
                statuses[returnId] = 'Declined';
                updatedDisabledCheckboxes[returnId] = true;
              } else {
                statuses[returnId] = returnDetail.status;
              }
            });
          } else {
            console.warn(`Expected an array but got ${typeof details}`);
          }
        } catch (error) {
          console.error(`Error parsing JSON for order ${order.id}:`, error);
        }
      });
      
      setReturnStatuses(statuses);
      setDisabledCheckboxes(updatedDisabledCheckboxes);

      setIsLoading(false);
      
     }
     catch (error){
      console.error('Error fetching data:', error);
     }
      
    };



    useEffect(() => {
      // Call the appropriate function based on selectedAPI value
      if (selectedAPI === 1) {
        fetchOrderDetails();
      } else {
        fetchOrderSkyexDetails();
      }
    }, [selectedAPI]);

    
    const closeModal = () => {
      setIsModalVisible(false);
      
    };

    const handleViewDetails = async (orderId) => {
      const selectedCompanyId = selectedCompanyIds[orderId];
      try{
        setLoadingRows(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: true
        }));
       

        const response = await fetch('/api/getAWBNumber',{
          method:'POST',
          headers:{
            'Content-Type':'application/json'
          },
          body: JSON.stringify({
            orderId: orderId , 
            selectedCompanyId:selectedCompanyId    
             }),
        });
        const data= await response.json();
        setLoadingRows(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: false
        }));
        setOrders(data.orders);
        setPushedOrders(data.pushed_orders);
        setRemovedOrders(data.removed_orders);
        
        if(response.ok)
        {
          setToastProps({
            content: ("Pushed to SkyEx successfully")
            
          });        
        
        }
        else{
          setToastProps({
						content: (data.error),
						error: true,
					});
          setOrders(data.orders);
          setPushedOrders(data.pushed_orders);
          setRemovedOrders(data.removed_orders);
        }
        if (data.error) {
          console.log(data.error);
          setToastProps({
						content: (data.error),
						error: true,
					});
          setOrders(data.orders);
          setPushedOrders(data.pushed_orders);
          setRemovedOrders(data.removed_orders);
        }
                
       }
       catch (error){
        console.error('Error fetching data:', error);
       }
    
    };


    const handleSkyExPush = async (orderId) => {
      const selectedCompanyId = selectedCompanyIds[orderId];
      try{
        setLoadingRows(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: true
        }));
       

        const response = await fetch('/api/skyExGetAWBNumber',{
          method:'POST',
          headers:{
            'Content-Type':'application/json'
          },
          body: JSON.stringify({
            orderId: orderId , 
            
             }),
        });
        const data= await response.json();
        setLoadingRows(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: false
        }));
        setOrders(data.orders);
        setPushedOrders(data.pushed_orders);
        setRemovedOrders(data.removed_orders);
        
        if(response.ok)
        {
          setToastProps({
            content: ("Pushed to SkyEx successfully")
            
          });        
        
        }
        else{
          setToastProps({
						content: (data.error),
						error: true,
					});
          setOrders(data.orders);
          setPushedOrders(data.pushed_orders);
          setRemovedOrders(data.removed_orders);
        }
        if (data.error) {
          console.log(data.error);
          setToastProps({
						content: (data.error),
						error: true,
					});
          setOrders(data.orders);
          setPushedOrders(data.pushed_orders);
          setRemovedOrders(data.removed_orders);
        }
                
       }
       catch (error){
        console.error('Error fetching data:', error);
       }
    
    };

    const handleReload = async (orderId) => {
      try {
        setReloadLoader(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: true
        }));
    
        const response = await fetch('/api/getReloadData', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json'
          },
          body: JSON.stringify({
            orderId: orderId
          }),
        });
    
        if (!response.ok) {
          throw new Error('Failed to reload.');
        }
    
        const result = await response.json();
        setReloadLoader(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: false
        }));
    
        if (result && result.updatedOrder) {
          console.log(result.updatedOrder);
          setOrders(prevOrderData => {
            return prevOrderData.map(order => {
              return order.order_id === result.updatedOrder.order_id ? { ...order, ...result.updatedOrder } : order;
            });
          });
          setToastProps({
            content: "Reload successful.",
            error: false,
          });
        } else {
          throw new Error('No updated order found or invalid response.');
        }
    
      } catch (error) {
        console.error('Error reloading:', error);
        setToastProps({
          content: "Failed to reload.",
          error: true,
        });
      } finally {
                setReloadLoader(
       {});
      }
    };
      

   


    const handleReloadRemove = async (orderId) => {
      try {
        setReloadLoaderRemove(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: true
        }));
    
        const response = await fetch('/api/getReloadData', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json'
          },
          body: JSON.stringify({
            orderId: orderId
          }),
        });
    
        if (!response.ok) {
          throw new Error('Failed to reload.');
        }
    
        const result = await response.json();
        setReloadLoaderRemove(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: false
        }));
    
        if (result && result.updatedOrder) {
          console.log(result.updatedOrder);
          setRemovedOrders(prevOrderData => {
            return prevOrderData.map(order => {
              return order.order_id === result.updatedOrder.order_id ? { ...order, ...result.updatedOrder } : order;
            });
          });
          setToastProps({
            content: "Reload successful.",
            error: false,
          });
        } else {
          throw new Error('No updated order found or invalid response.');
        }
    
      } catch (error) {
        console.error('Error reloading:', error);
        setToastProps({
          content: "Failed to reload.",
          error: true,
        });
      } finally {
        setReloadLoaderRemove({});
      }
    };
     
    const handleDownload = useCallback(async (orderId) => {
      console.log(orderId);
      if (activeDownloads.current[orderId]) return; 
  
      activeDownloads.current[orderId] = true;
      setLoadingRowsDownload((prevLoadingRows) => ({
        ...prevLoadingRows,
        [orderId]: true,
      }));
  
      try {
        const response = await fetch('/api/getDownloadLabel', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({ orderId }),
        });
  
        const data = await response.json();
  
        if (response.ok) {
          setToastProps({ content: 'Downloaded successfully' });
  
          if (data.awb_link) {
            window.open(data.awb_link, '_blank');
          }
        } else {
          console.error('Failed to fetch data:', data.error);
          setToastProps({ content: data.error, error: true });
        }
      } catch (error) {
        console.error('Error fetching data:', error);
      } finally {
        setLoadingRowsDownload((prevLoadingRows) => ({
          ...prevLoadingRows,
          [orderId]: false,
        }));
        activeDownloads.current[orderId] = false; 
      }
    }, []);

    // const handleDownloadSkyEx = useCallback(async (orderId) => {
    //   console.log(orderId);
    //   if (activeDownloads.current[orderId]) return; 
  
    //   activeDownloads.current[orderId] = true;
    //   setLoadingRowsDownload((prevLoadingRows) => ({
    //     ...prevLoadingRows,
    //     [orderId]: true,
    //   }));
  
    //   try {
    //     const response = await fetch('/api/getDownloadLabelSkyEx', {
    //       method: 'POST',
    //       headers: {
    //         'Content-Type': 'application/json',
    //       },
    //       body: JSON.stringify({ orderId }),
    //     });
  
    //    // const data = await response.json();
  
    //     if (response.ok) {
    //      // setToastProps({ content: 'Downloaded successfully' });
    //      // console.log(data.awb_link);
    //      // if (data.awb_link) {


    //         const blob = await response.blob();
    //         const url = window.URL.createObjectURL(blob);
      
    //         setToastProps({
    //             content: "Downloaded successfully",
    //         });
      
            
    //         const a = document.createElement('a');
    //         a.style.display = 'none';
    //         a.href = url;
    //         a.download = 'awb_label.pdf'; 
    //         document.body.appendChild(a);
    //         a.click();
    //         window.URL.revokeObjectURL(url);
    //         document.body.removeChild(a);

            
    //        //window.open(data.awb_link, '_blank');
    //        //window.location.href = data.awb_link;
    //     //     const link = document.createElement('a');
    //     // link.href = data.awb_link;
    //     // link.download = 'your-pdf-file.pdf'; // Optional filename for the download
    //     // document.body.appendChild(link);
    //     // link.click();
    //     // document.body.removeChild(link);
    //      // }
    //     } else {
    //       console.error('Failed to fetch data:', data.error);
    //       setToastProps({ content: data.error, error: true });
    //     }
      

    //   } catch (error) {
    //     console.error('Error fetching data:', error);
    //   } finally {
    //     setLoadingRowsDownload((prevLoadingRows) => ({
    //       ...prevLoadingRows,
    //       [orderId]: false,
    //     }));
    //     activeDownloads.current[orderId] = false; 
    //   }
    // }, []);


    const handleDownloadSkyEx = useCallback(async (orderId) => {
      console.log(`Downloading for order: ${orderId}`);
      if (activeDownloads.current[orderId]) return; 
    
      activeDownloads.current[orderId] = true;
      setLoadingRowsDownload((prevLoadingRows) => ({
        ...prevLoadingRows,
        [orderId]: true,
      }));
    
      try {
        const response = await fetch('/api/getDownloadLabelSkyEx', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({ orderId }),
        });
    
        if (!response.ok) {
          throw new Error('Failed to fetch PDF.');
        }
    
        const blob = await response.blob();
    
        const contentDisposition = response.headers.get('Content-Disposition');
        console.log('Content-Disposition:', contentDisposition);
    
        let filename = 'awb_label.pdf';
        if (contentDisposition) {
          const filenameMatch = contentDisposition.match(/filename="?(.+)"?/i);
          if (filenameMatch) {
            filename = filenameMatch[1];
            console.log('Extracted filename:', filename);
            filename = filename.replace(/_+$/, '');
            console.log('Cleaned filename:', filename);
          }
        }
    
        const pdfBlob = new Blob([blob], { type: 'application/pdf' });
        const url = window.URL.createObjectURL(pdfBlob);
    
        console.log('Final filename for download:', filename);
    
        const a = document.createElement('a');
        a.style.display = 'none';
        a.href = url;
        a.download = filename;
        document.body.appendChild(a);
        a.click();
        window.URL.revokeObjectURL(url);
        document.body.removeChild(a);
    
        setToastProps({
          content: "Downloaded successfully",
        });
    
      } catch (error) {
        console.error('Error downloading label:', error);
        setToastProps({
          content: "Error downloading label",
          error: true,
        });
      } finally {
        setLoadingRowsDownload((prevLoadingRows) => ({
          ...prevLoadingRows,
          [orderId]: false,
        }));
        activeDownloads.current[orderId] = false; 
      }
    }, []);


    const handleReturnPushToSkyex = async (orderId) => {
      //console.log(orderId);
      try{
        setLoadingRows(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: true
        }));
        const response = await fetch('/api/getAWBNumberReturn',{
          method:'POST',
          headers:{
            'Content-Type':'application/json'
          },
          body: JSON.stringify({
            orderId: orderId       
             }),
        });
        const data= await response.json();
        setLoadingRows(prevLoadingRows => ({
          ...prevLoadingRows,
          [orderId]: false
        }));
        setOrders(data.orders);
        setPushedOrders(data.pushed_orders);
        setRemovedOrders(data.removed_orders);
       

        if(response.ok)
        {
          setToastProps({
            content: ("Pushed to SkyEx successfully")
            
          });      
          fetchOrderDetails();  
          fetchOrderSkyexDetails();
        }
        else{
          setToastProps({
						content: (data.error),
						error: true,
					});
          setOrders(data.orders);
          setPushedOrders(data.pushed_orders);
          setRemovedOrders(data.removed_orders);
        }
        if (data.error) {
          console.log(data.error);
          setToastProps({
						content: (data.error),
						error: true,
					});
          setOrders(data.orders);
          setPushedOrders(data.pushed_orders);
          setRemovedOrders(data.removed_orders);
        }
                
       }
       catch (error){
        console.error('Error fetching data:', error);
       }
    
    };

    useEffect(() => {
      
      fetchOrderDetails();
      fetchOrderSkyexDetails();
     
    }, []); 

    const [selectedTab, setSelectedTab] = useState(0);

    const handleTabChange = useCallback((selectedTabIndex) => {
      setSelectedTab(selectedTabIndex);
    }, []);

    // const {selectedResources, allResourcesSelected, handleSelectionChange} =
    // useIndexResourceState(orderData);

    // const {selectedResourcesPush, allResourcesSelectedPush, handleSelectionChangePush} =
    // useIndexResourceState(pushedOrder);
    const {
      selectedResources,
      allResourcesSelected,
      handleSelectionChange
    } = useIndexResourceState(orderData);
    
    const {
      selectedResources: selectedResourcesPush,
      allResourcesSelected: allResourcesSelectedPush,
      handleSelectionChange: handleSelectionChangePush
    } = useIndexResourceState(pushedOrder);
    
    const {
      selectedResources: selectedResourcesRemove,
      allResourcesSelected: allResourcesSelectedRemove,
      handleSelectionChange: handleSelectionChangeRemove
    } = useIndexResourceState(removedOrders);

    const {
      selectedResources: selectedResourcesReturn,
      allResourcesSelected: allResourcesSelectedReturn,
      handleSelectionChange: handleSelectionChangeReturn
    } = useIndexResourceState(returnedOrders);

    const [expandedRowId, setExpandedRowId] = useState(null);
    const toggleRow = (orderId) => {
      setExpandedRowId(prevId => prevId === orderId ? null : orderId);
    };

      // const {
      //   selectedResources: selectedReturnLineItems,
      //   allResourcesSelected: allReturnLineItemsSelected,
      //   handleSelectionChange: handleReturnLineItemSelectionChange
      // } = useIndexResourceState([]);
     
      
    const resourceNames = {
      singular: 'order',
      plural: 'orders',
    };
    
    const tabs = [
      {
        id: 'new-order',
        content: `New Order (${orderData.length})`,
        panelID: 'new-order-panel',
        renderContent: () => (
            <LegacyCard.Section>
              <div style={{
                display: 'flex',
                justifyContent: 'end'
              }}> 
              <Button
                onClick={() => {
                  selectedAPI === 1 ? fetchOrderDetails() : fetchOrderSkyexDetails();
                }}
              >
                Get New Orders
              </Button>

              </div>
           
    								<IndexFilters
									sortOptions={sortOptions}
									sortSelected={sortSelected}
									queryValue={queryValue}
									queryPlaceholder="Search"
									onQueryChange={handleFiltersQueryChange}
									onQueryClear={() => setQueryValue('')}
									onSort={setSortSelected}
									cancelAction={{
										onAction: onHandleCancel,
										disabled: false,
										loading: false,
									}}
									tabs={indexTabs}
									selected={selected}
									onSelect={setSelected}
									canCreateNewView
									filters={filters}
									appliedFilters={appliedFilters}
									onClearAll={handleFiltersClearAll}
									mode={mode}
									setMode={setMode}
									loading={isLoading}
								/>

								{orderData && (
									<IndexTable
										resourceName={resourceNames}
										itemCount={orderData.length}
										headings={[
											{ title: 'Sl.No' },
											{ title: 'Date' },
											{ title: 'Order' },
											{ title: 'Customer' },
											{ title: 'Total' },
											{ title: 'Payment status' },
                      ...(selectedAPI === 1 ? [{ title: 'Status' }] : []),
                      ...(selectedAPI === 1 ? [{ title: 'Location' }] : []),
											{ title: 'Generate AWB & Label' },
                      ...(selectedAPI === 1 ? [{ title: 'Reload Data' }] : []),
										]}
										//selectable={false}
                    promotedBulkActions={promotedBulkActions}
                    selectedItemsCount={
                      allResourcesSelected ? 'All' : selectedResources.length
                    }
                    onSelectionChange={handleSelectionChange}
									>
										{rowMarkup}
									</IndexTable>
								)}
               <div style={{ float: 'right' }}>
									<Pagination
										hasPrevious={currentPage > 1}
										onPrevious={handlePrevious}
										hasNext={(currentPage * itemsPerPage) < filteredOrder?.length}
										onNext={handleNext}
									/>

								</div>
         
          </LegacyCard.Section>
          
        )
      },
      {
        id: 'pushed-order',
        content: `Pushed Order (${pushedOrder.length})`,
        panelID: 'pushed-order-panel',
        renderContent: () => (
          <LegacyCard.Section>
             <div style={{
                display: 'flex',
                justifyContent: 'end'
              }}> 
              <Button
                onClick={() => {
                  selectedAPI === 1 ? fetchOrderDetails() : fetchOrderSkyexDetails();
                }}
              >
                Reload Data
              </Button>
              
              </div>
           	<IndexFilters
									sortOptions={sortOptions}
									sortSelected={sortSelected}
									queryValue={queryValue}
									queryPlaceholder="Search"
									onQueryChange={handleFiltersQueryChange}
									onQueryClear={() => setQueryValue('')}
									onSort={setSortSelected}
									cancelAction={{
										onAction: onHandleCancel,
										disabled: false,
										loading: false,
									}}
									tabs={indexTabs}
									selected={selected}
									onSelect={setSelected}
									canCreateNewView
									filters={filters}
									appliedFilters={appliedFilters}
									onClearAll={handleFiltersClearAll}
									mode={mode}
									setMode={setMode}
									loading={isLoading}
								/>

								{pushedOrder && (
									<IndexTable
										resourceName={resourceNames}
										itemCount={pushedOrder.length}
										headings={[
											{ title: 'Sl.No' },
											{ title: 'Date' },
											{ title: 'Order' },
											{ title: 'Customer' },
											{ title: 'Total' },
											{ title: 'Payment status' },
                      { title: 'AWB No' },
                      { title: 'Status' },
											{ title: 'Label' }
										]}
										
                    promotedBulkActions={promotedBulkActionsPush}
                  selectedItemsCount={
                  allResourcesSelectedPush ? 'All' : selectedResourcesPush.length
                }
                onSelectionChange={handleSelectionChangePush}
									>
										{pushedRowMarkup}
									</IndexTable>
								)}
               <div style={{ float: 'right' }}>
									<Pagination
										hasPrevious={pushedCurrentPage > 1}
										onPrevious={handlePushPrevious}
										hasNext={(pushedCurrentPage * itemsPerPage) < pushedFilteredOrder?.length}
										onNext={handlePushNext}
									/>

								</div>
          			
          </LegacyCard.Section>
        )
      },
      {
        id: 'removed-lists',
        content: `Removed Lists (${removedOrders.length})`,
        panelID: 'removed-lists-panel',
        renderContent: () => (
          <LegacyCard.Section>
           	<IndexFilters
									sortOptions={sortOptions}
									sortSelected={sortSelected}
									queryValue={queryValue}
									queryPlaceholder="Search"
									onQueryChange={handleFiltersQueryChange}
									onQueryClear={() => setQueryValue('')}
									onSort={setSortSelected}
									cancelAction={{
										onAction: onHandleCancel,
										disabled: false,
										loading: false,
									}}
									tabs={indexTabs}
									selected={selected}
									onSelect={setSelected}
									canCreateNewView
									filters={filters}
									appliedFilters={appliedFilters}
									onClearAll={handleFiltersClearAll}
									mode={mode}
									setMode={setMode}
									loading={isLoading}
								/>

								{removedOrders && (
									<IndexTable
										resourceName={resourceNames}
										itemCount={removedOrders.length}
										headings={[
											{ title: 'Sl.No' },
											{ title: 'Date' },
											{ title: 'Order' },
											{ title: 'Customer' },
											{ title: 'Total' },
											{ title: 'Payment status' },
                      ...(selectedAPI === 1 ? [{ title: 'Status' }] : []),
                      ...(selectedAPI === 1 ? [{ title: 'Location' }] : []),                 
											{ title: 'Generate AWB & Label' },
                      ...(selectedAPI === 1 ? [{ title: 'Reload Data' }] : []),
										]}
										//selectable={false}
                    promotedBulkActions={promotedBulkActionsRemove}
                    selectedItemsCount={
                    allResourcesSelectedRemove ? 'All' : selectedResourcesRemove.length
                  }
                  onSelectionChange={handleSelectionChangeRemove}
									>
										{rowMarkupRemove}
									</IndexTable>
								)}
               <div style={{ float: 'right' }}>
									<Pagination
										hasPrevious={currentPageRemove > 1}
										onPrevious={handlePreviousRemove}
										hasNext={(currentPageRemove * itemsPerPage) < filteredOrderRemove?.length}
										onNext={handleNextRemove}
									/>

								</div>
							            
          </LegacyCard.Section>
        )
      },
      {
        id: 'return-requests',
        content: `Return Requests (${returnedOrders.length})`,
        panelID: 'return-requests-panel',
        renderContent: () => (
          <LegacyCard.Section>
             <div style={{
                display: 'flex',
                justifyContent: 'end'
              }}> 
              <Button
                onClick={() => {
                  selectedAPI === 1 ? fetchOrderDetails() : fetchOrderSkyexDetails();
                }}
              >
                Get New Requests
              </Button>
             
              </div>

           	<IndexFilters
									sortOptions={sortOptions}
									sortSelected={sortSelected}
									queryValue={queryValue}
									queryPlaceholder="Search"
									onQueryChange={handleFiltersQueryChange}
									onQueryClear={() => setQueryValue('')}
									onSort={setSortSelected}
									cancelAction={{
										onAction: onHandleCancel,
										disabled: false,
										loading: false,
									}}
									tabs={indexTabs}
									selected={selected}
									onSelect={setSelected}
									canCreateNewView
									filters={returnFilters}
									appliedFilters={appliedFilters}
									onClearAll={handleFiltersClearAll}
									mode={mode}
									setMode={setMode}
									loading={isLoading}
								/>

								{returnedOrders && (
									<IndexTable
										resourceName={resourceNames}
										itemCount={returnedOrders.length}
										headings={[
                      { title: '' },
											{ title: 'Sl.No' },
											{ title: 'Date' },
											{ title: 'Order' },
											{ title: 'Customer' },
											{ title: 'Total' },
											{ title: 'Return status' },
											{ title: 'Generate AWB & Label' }
										]}
										selectable={false}
                   // promotedBulkActions={promotedBulkActionsReturnProducts}
                    selectedItemsCount={
                    allResourcesSelectedReturn ? 'All' : selectedResourcesReturn.length
                  }
                  onSelectionChange={handleSelectionChangeReturn}
									>
										{rowMarkupReturn}
									</IndexTable>
								)}
                <div style={{ float: 'right' }}>
									<Pagination
										hasPrevious={currentPageReturn > 1}
										onPrevious={handlePreviousReturn}
										hasNext={(currentPageReturn * itemsPerPage) < returnedFilteredOrder?.length}
										onNext={handleNextReturn}
									/>

								</div>
							            
          </LegacyCard.Section>
        )
      }

    ];

    const sortOptions = [];
    const { mode, setMode } = useSetIndexFiltersMode(IndexFiltersMode.Filtering);
    const [sortSelected, setSortSelected] = useState(['date asc']);
    const [queryValue, setQueryValue] = useState('');
    const handleAccountStatusRemove = useCallback(() => setAccountStatus(), []);
       
    const [filteredOrder, setFilteredOrder] = useState([]);
    const [pushedFilteredOrder, setPushedFilteredOrder] = useState([]);
    const [filteredOrderRemove, setFilteredOrderRemove] = useState([]);
    const [returnedFilteredOrder, setReturnedFilteredOrder] = useState([]);
   
    // Handling filter actions
  
    const [paymentStatus, setPaymentStatus] = useState(undefined);
    const [returnStatus, setReturnStatus] = useState(undefined);
    const [accountStatus, setAccountStatus] = useState(undefined);
  
    // Date filtering
    const [dateStatus, setDateStatus] = useState(undefined);
    const currentDate = new Date();
    const [{ month, year }, setDate] = useState({
      month: currentDate.getMonth(),
      year: currentDate.getFullYear(),
    });
    const [selectedDates, setSelectedDates] = useState({
      start: currentDate,
      end: currentDate,
    });
  
    const handleMonthChange = useCallback(
      (month, year) => setDate({ month, year }),
      [],
    );
    const handlePaymentStatusChange = useCallback((value) => {
  
      setPaymentStatus(value);
  
    }, []);

    const handleReturnStatusChange = useCallback((value) => {
  
      setReturnStatus(value);
  
    }, []);

    const handleAccountStatusChange = useCallback((value) => {
  
      setAccountStatus(value);
  
    }, []);
  
    const handleVendorChange = useCallback((value) => {
      setProductVendors(value);
    }, []);
  
    const handleDateStatusChange = useCallback((value) => {
      setDateStatus(value);
    }, []);
  
  
    const handleFiltersQueryChange = useCallback(
      (value) =>
        setQueryValue(value),
      // build query
      [],
    );
    const handlePaymentStatusRemove = useCallback(
      () => setPaymentStatus(undefined),
      [],
    );
    const handleReturnStatusRemove = useCallback(
      () => setReturnStatus(undefined),
      [],
    );
    const handleStatusRemove = useCallback(
      () => setAccountStatus(undefined),
      [],
    );
  
    
    const handleDateRemove = useCallback(
      () => setDateStatus(undefined),
      [],
    )
    const onHandleCancel = () => { };
    const handleQueryValueRemove = useCallback(() => setQueryValue(''), []);
  
    const handleFiltersClearAll = useCallback(() => {
      handlePaymentStatusRemove();
      handleStatusRemove();
      handleQueryValueRemove();
      handleDateRemove();
      handleReturnStatusRemove();
    }, [
      handlePaymentStatusRemove,
      handleStatusRemove,
      handleQueryValueRemove,
      handleDateRemove,
      handleReturnStatusRemove
      
    ]);
  
    // Select filter show the bar
    const appliedFilters = [];
  
    if (returnStatus && !isEmpty(returnStatus)) {
      const key = 'returnStatus';
      appliedFilters.push({
        key,
        label: disambiguateLabel(key, returnStatus),
        onRemove: handleAccountStatusRemove,
      });
    }
    if (paymentStatus && !isEmpty(paymentStatus)) {
      const key = 'paymentStatus';
      appliedFilters.push({
        key,
        label: disambiguateLabel(key, paymentStatus),
        onRemove: handlePaymentStatusRemove,
      });
    }
    if (dateStatus && !isEmpty(dateStatus)) {
      const key = 'dateStatus';
      appliedFilters.push({
        key,
        label: disambiguateLabel(key, dateStatus),
        onRemove: handleDateRemove,
      });
    }
    if (accountStatus && !isEmpty(accountStatus)) {
      const key = 'accountStatus';
      appliedFilters.push({
        key,
        label: disambiguateLabel(key, accountStatus),
        onRemove: handleStatusRemove,
      });
    }
  
    const accountStatusChoices = [
      { label: 'Created', value: 'Created' },
      { label: 'Pending', value: 'Pending' },
      { label: 'Ready For Pickup', value: 'Ready For Pickup' },
    ];
    const mapAccountStatusToValue = (status) => {
      switch (status) {
        case 'Created':
          return 'Created';
        case 'Pending':
          return 'Pending';
        case 'Ready For Pickup':
          return 'Ready For Pickup';
        default:
          return status;
      }
    };
  
    const dateStatusChoices = [
      { label: 'Today', value: 'Today' },
      { label: 'Last 7 days', value: 'Last 7 days' },
      { label: 'Last 30 days', value: 'Last 30 days' },
      { label: 'Last 90 days', value: 'Last 90 days' },
      { label: 'Last 12 months', value: 'Last 12 months' },
      { label: 'Custom Date', value: 'Custom Date' },
    ];
    const paymentStatusChoices = [
      { label: 'Paid', value: 'paid' },
      { label: 'Pending', value: 'pending' },
      { label: 'Partially Paid', value: 'partially_paid' },
      { label: 'Partially Refunded', value: 'partially_refunded' },
      { label: 'Refunded', value: 'refunded' },
  
    ];
    const returnStatusChoices = [
      { label: 'Approved', value: 'approved' },
      { label: 'Declined', value: 'declined' },
      { label: 'Partially approved', value: 'partially approved' },
      { label: 'Pending', value: 'pending' }, 
    ];

    const filters = [
  
      {
        key: 'paymentStatus',
        label: 'Payment Status',
        filter: (
          <ChoiceList
            title="Payment status"
            titleHidden
            choices={paymentStatusChoices}
            selected={paymentStatus || []}
            onChange={handlePaymentStatusChange}
            allowMultiple
  
          />
        ),
        shortcut: true,
      },
  
      // {
      //   key: 'accountStatus',
      //   label: 'Status',
      //   filter: (
      //     <ChoiceList
      //       title="Status"
      //       titleHidden
      //       choices={accountStatusChoices}
      //       selected={accountStatus || []}
      //       onChange={handleAccountStatusChange}
      //       allowMultiple
  
      //     />
      //   ),
      //   shortcut: true,
      // },
  
     
      {
        key: 'dateStatus',
        label: 'Date',
        filter: (
          <>
            <ChoiceList
              title="Date status"
              titleHidden
              choices={dateStatusChoices}
              selected={dateStatus || []}
              onChange={handleDateStatusChange}
            />
            {dateStatus?.includes('Custom Date') && (
              <DatePicker
                month={month}
                year={year}
                onChange={setSelectedDates}
                onMonthChange={handleMonthChange}
                selected={selectedDates}
                allowRange
              />
            )}
          </>
        ),
        shortcut: true,
      },
    ];
  
    const returnFilters = [
  
      {
        key: 'returnStatus',
        label: 'Return Status',
        filter: (
          <ChoiceList
            title="Return status"
            titleHidden
            choices={returnStatusChoices}
            selected={returnStatus || []}
            onChange={handleReturnStatusChange}
            allowMultiple
  
          />
        ),
        shortcut: true,
      },
  
      {
        key: 'dateStatus',
        label: 'Date',
        filter: (
          <>
            <ChoiceList
              title="Date status"
              titleHidden
              choices={dateStatusChoices}
              selected={dateStatus || []}
              onChange={handleDateStatusChange}
            />
            {dateStatus?.includes('Custom Date') && (
              <DatePicker
                month={month}
                year={year}
                onChange={setSelectedDates}
                onMonthChange={handleMonthChange}
                selected={selectedDates}
                allowRange
              />
            )}
          </>
        ),
        shortcut: true,
      },
    ];

    useEffect(() => {
      if (orderData) {
        setCurrentPage(1);
  
        const lowercasedQuery = queryValue?.toLowerCase() || "";
        const searchedOrder = orderData.filter(order => {
          const customer = order.order_data.customer;
          const nameMatch = lowercasedQuery
            ? (customer && customer.first_name && customer.first_name.toLowerCase().includes(lowercasedQuery))
            : true;
  
          let dateMatch = true;
          if (dateStatus) {
            if (dateStatus.includes('Custom Date')) {
              const startDate = selectedDates.start;
              const endDate = new Date(selectedDates.end);
              startDate.setHours(0, 0, 0, 0);
              endDate.setHours(23, 59, 59, 999);
  
              const orderDate = new Date(order.order_data.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            } else {
              const startDate = new Date();
              const endDate = new Date();
              endDate.setHours(23, 59, 59, 999);
              switch (dateStatus[0]) {
                case 'Last 7 days':
                  startDate.setDate(startDate.getDate() - 7);
                  break;
                case 'Last 30 days':
                  startDate.setDate(startDate.getDate() - 30);
                  break;
                case 'Last 90 days':
                  startDate.setDate(startDate.getDate() - 90);
                  break;
                case 'Last 12 months':
                  startDate.setFullYear(startDate.getFullYear() - 1);
                  break;
                default:
              }
  
              startDate.setHours(0, 0, 0, 0);
              const orderDate = new Date(order.order_data.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            }
          }
  
          const paymentMatch =
            paymentStatus && paymentStatus.length > 0
              ? paymentStatus.some(payment =>
                order.order_data.financial_status.toLowerCase().includes(payment.toLowerCase())
              )
              : true;
  
          const statusMatch =
            accountStatus && accountStatus.length > 0
              ? accountStatus.some(statusLabel => {
                const statusValue = mapAccountStatusToValue(statusLabel);
                if (statusValue === 'null') {
                  return !order.order_data.fulfillment_status;
                } else {
                  return (
                    order.order_data.fulfillment_status &&
                    order.order_data.fulfillment_status.toLowerCase().includes(statusValue.toLowerCase())
                  );
                }
              })
              : true;
  
          return statusMatch && paymentMatch && (nameMatch) && dateMatch;
        });
  
        setFilteredOrder(searchedOrder);
      }

      if (pushedOrder) {
        setPushedCurrentPage(1);
  
        const lowercasedQuery = queryValue?.toLowerCase() || "";
        const searchedOrder = pushedOrder.filter(order => {
          const customer = order.order_data.customer;
          const nameMatch = lowercasedQuery
            ? (customer && customer.first_name && customer.first_name.toLowerCase().includes(lowercasedQuery))
            : true;
  
          let dateMatch = true;
          if (dateStatus) {
            if (dateStatus.includes('Custom Date')) {
              const startDate = selectedDates.start;
              const endDate = new Date(selectedDates.end);
              startDate.setHours(0, 0, 0, 0);
              endDate.setHours(23, 59, 59, 999);
  
              const orderDate = new Date(order.order_data.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            } else {
              const startDate = new Date();
              const endDate = new Date();
              endDate.setHours(23, 59, 59, 999);
              switch (dateStatus[0]) {
                case 'Last 7 days':
                  startDate.setDate(startDate.getDate() - 7);
                  break;
                case 'Last 30 days':
                  startDate.setDate(startDate.getDate() - 30);
                  break;
                case 'Last 90 days':
                  startDate.setDate(startDate.getDate() - 90);
                  break;
                case 'Last 12 months':
                  startDate.setFullYear(startDate.getFullYear() - 1);
                  break;
                default:
              }
  
              startDate.setHours(0, 0, 0, 0);
              const orderDate = new Date(order.order_data.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            }
          }
  
          const paymentMatch =
            paymentStatus && paymentStatus.length > 0
              ? paymentStatus.some(payment =>
                order.order_data.financial_status.toLowerCase().includes(payment.toLowerCase())
              )
              : true;
  
          const statusMatch =
            accountStatus && accountStatus.length > 0
              ? accountStatus.some(statusLabel => {
                const statusValue = mapAccountStatusToValue(statusLabel);
                if (statusValue === 'null') {
                  return !order.order_data.fulfillment_status;
                } else {
                  return (
                    order.order_data.fulfillment_status &&
                    order.order_data.fulfillment_status.toLowerCase().includes(statusValue.toLowerCase())
                  );
                }
              })
              : true;
  
          return statusMatch && paymentMatch && (nameMatch) && dateMatch;
        });
  
        setPushedFilteredOrder(searchedOrder);
      }
      if (removedOrders) {
        setCurrentPageRemove(1);
  
        const lowercasedQuery = queryValue?.toLowerCase() || "";
        const searchedOrder = removedOrders.filter(order => {
          const customer = order.order_data.customer;
          const nameMatch = lowercasedQuery
            ? (customer && customer.first_name && customer.first_name.toLowerCase().includes(lowercasedQuery))
            : true;
  
          let dateMatch = true;
          if (dateStatus) {
            if (dateStatus.includes('Custom Date')) {
              const startDate = selectedDates.start;
              const endDate = new Date(selectedDates.end);
              startDate.setHours(0, 0, 0, 0);
              endDate.setHours(23, 59, 59, 999);
  
              const orderDate = new Date(order.order_data.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            } else {
              const startDate = new Date();
              const endDate = new Date();
              endDate.setHours(23, 59, 59, 999);
              switch (dateStatus[0]) {
                case 'Last 7 days':
                  startDate.setDate(startDate.getDate() - 7);
                  break;
                case 'Last 30 days':
                  startDate.setDate(startDate.getDate() - 30);
                  break;
                case 'Last 90 days':
                  startDate.setDate(startDate.getDate() - 90);
                  break;
                case 'Last 12 months':
                  startDate.setFullYear(startDate.getFullYear() - 1);
                  break;
                default:
              }
  
              startDate.setHours(0, 0, 0, 0);
              const orderDate = new Date(order.order_data.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            }
          }
  
          const paymentMatch =
            paymentStatus && paymentStatus.length > 0
              ? paymentStatus.some(payment =>
                order.order_data.financial_status.toLowerCase().includes(payment.toLowerCase())
              )
              : true;
  
          const statusMatch =
            accountStatus && accountStatus.length > 0
              ? accountStatus.some(statusLabel => {
                const statusValue = mapAccountStatusToValue(statusLabel);
                if (statusValue === 'null') {
                  return !order.order_data.fulfillment_status;
                } else {
                  return (
                    order.order_data.fulfillment_status &&
                    order.order_data.fulfillment_status.toLowerCase().includes(statusValue.toLowerCase())
                  );
                }
              })
              : true;
  
          return statusMatch && paymentMatch && (nameMatch) && dateMatch;
        });
  
        setFilteredOrderRemove(searchedOrder);
      }
      if (returnedOrders) {
        setCurrentPageReturn(1);
        
        const lowercasedQuery = queryValue?.toLowerCase() || "";
        const searchedOrder = returnedOrders.filter(order => {
          const customer = order.customer_first_name;
        
          const nameMatch = lowercasedQuery
            ? (customer && customer.toLowerCase().includes(lowercasedQuery))
            : true;
            
          let dateMatch = true;
          if (dateStatus) {
            if (dateStatus.includes('Custom Date')) {
              const startDate = selectedDates.start;
              const endDate = new Date(selectedDates.end);
              startDate.setHours(0, 0, 0, 0);
              endDate.setHours(23, 59, 59, 999);
  
              const orderDate = new Date(order.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            } else {
              const startDate = new Date();
              const endDate = new Date();
              endDate.setHours(23, 59, 59, 999);
              switch (dateStatus[0]) {
                case 'Last 7 days':
                  startDate.setDate(startDate.getDate() - 7);
                  break;
                case 'Last 30 days':
                  startDate.setDate(startDate.getDate() - 30);
                  break;
                case 'Last 90 days':
                  startDate.setDate(startDate.getDate() - 90);
                  break;
                case 'Last 12 months':
                  startDate.setFullYear(startDate.getFullYear() - 1);
                  break;
                default:
              }
  
              startDate.setHours(0, 0, 0, 0);
              const orderDate = new Date(order.created_at);
              dateMatch = orderDate >= startDate && orderDate <= endDate;
            }
          }
  
          const returnMatch =
          returnStatus && returnStatus.length > 0
            ? returnStatus.some(status => {
                let overallStatus = 'Approved';
                let hasApproved = false;
                let hasDeclined = false;
                let hasPending = false;
        
                JSON.parse(order.returnLineItem_details).forEach((item, index) => {
                  const returnDetails = JSON.parse(order.return_details);
                  const fullReturnId = returnDetails[index]?.id || '';
                  const returnId = fullReturnId.split('/').pop() || 'N/A';
                  const itemStatus = returnStatuses[returnId];
        
                  if (itemStatus === 'Approved') {
                    hasApproved = true;
                  } else if (itemStatus === 'Declined') {
                    hasDeclined = true;
                  } else {
                    hasPending = true;
                  }
                });
        
                if (hasPending) {
                  overallStatus = 'Pending';
                } else if (hasApproved && hasDeclined) {
                  overallStatus = 'Partially approved';
                } else if (hasDeclined) {
                  overallStatus = 'Declined';
                }
        
                return overallStatus.toLowerCase() === status.toLowerCase();
              })
            : true;
              //console.log(returnMatch);
          const statusMatch =
            accountStatus && accountStatus.length > 0
              ? accountStatus.some(statusLabel => {
                const statusValue = mapAccountStatusToValue(statusLabel);
                if (statusValue === 'null') {
                  return !order.order_data.fulfillment_status;
                } else {
                  return (
                    order.order_data.fulfillment_status &&
                    order.order_data.fulfillment_status.toLowerCase().includes(statusValue.toLowerCase())
                  );
                }
              })
              : true;
  
          return statusMatch && returnMatch && (nameMatch) && dateMatch;
        });
  
        //console.log(searchedOrder);
        setReturnedFilteredOrder(searchedOrder);
        
      }
    }, [orderData, accountStatus, queryValue, paymentStatus,dateStatus, selectedDates,pushedOrder,removedOrders,returnStatus,returnedOrders]);
  
  
    // Pagination using filtered products 
    useEffect(() => {
      const start = (currentPage - 1) * itemsPerPage;
      const end = start + itemsPerPage;
      if (filteredOrder)
        setPaginatedOrders(filteredOrder.slice(start, end));	

      const pushStart = (pushedCurrentPage - 1) * itemsPerPage;
      const pushEnd = pushStart + itemsPerPage;

      if (pushedFilteredOrder)
        setPushedPaginatedOrders(pushedFilteredOrder.slice(pushStart, pushEnd));

      const removeStart = (currentPageRemove - 1) * itemsPerPage;
      const removeEnd = removeStart + itemsPerPage;
      if (filteredOrder)
        setPaginatedOrderRemove(filteredOrderRemove.slice(removeStart, removeEnd));	

      const returnStart = (currentPageReturn - 1) * itemsPerPage;
      const returnEnd = returnStart + itemsPerPage;
      //console.log(returnedFilteredOrder);
      if (returnedFilteredOrder)
        setPaginatedOrderReturn(returnedFilteredOrder.slice(returnStart, returnEnd));	

    }, [currentPage, filteredOrder,pushedFilteredOrder,pushedCurrentPage,currentPageRemove,filteredOrderRemove,currentPageReturn,returnedFilteredOrder]);
  
    // Next and previous page
    const handlePrevious = () => {
      setCurrentPage(currentPage - 1);
    };
    const handleNext = () => {
      setCurrentPage(currentPage + 1);
    };
  
    // Next and previous page
    const handlePushPrevious = () => {
      setPushedCurrentPage(pushedCurrentPage - 1);
    };
    const handlePushNext = () => {
      setPushedCurrentPage(pushedCurrentPage + 1);
    };
  // Next and previous page
  const handlePreviousRemove = () => {
    setCurrentPageRemove(currentPageRemove - 1);
  };
  const handleNextRemove = () => {
    setCurrentPageRemove(currentPageRemove + 1);
  };
  // Next and previous page
  const handlePreviousReturn = () => {
    setCurrentPageReturn(currentPageReturn - 1);
  };
  const handleNextReturn = () => {
    setCurrentPageReturn(currentPageReturn + 1);
  };
    const rowMarkup = paginatedOrders?.map((order, index) => {

      const uniqueVendors = new Set();
  
      return (
      
        <IndexTable.Row
          id={order.id}
          key={order.id}
         selected={selectedResources.includes(order.id)}
          position={index}
          onClick={(e) => {
            e.stopPropagation(); 
          }}
        >
          <IndexTable.Cell>
            {((currentPage - 1) * itemsPerPage) + (index + 1)}
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {new Date(order.order_data.created_at).toLocaleDateString(('en-GB'), {
                year: 'numeric',
                month: '2-digit',
                day: '2-digit',
              })}
            </Text>
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {order.order_data.order_number}
            </Text>
          </IndexTable.Cell>
  
          <IndexTable.Cell>
  {order.order_data.customer?.first_name} {order.order_data.customer?.last_name}
</IndexTable.Cell>
  
           
          <IndexTable.Cell>{order.order_data.currency} {order.order_data.current_total_price}</IndexTable.Cell>
          <IndexTable.Cell>
            {order.order_data.financial_status === "pending"
              ? <Badge progress="incomplete" tone="attention">Pending</Badge>
              : order.order_data.financial_status === "paid"
                ? <Badge tone="success" progress="complete">Paid</Badge>
                : order.order_data.financial_status === "partially_paid"
                  ? <Badge progress="partiallyComplete" tone="warning">Partially paid</Badge>
                  : order.order_data.financial_status === "partially_refunded"
                    ? <Badge progress="partiallyComplete" tone="critical">Partially refunded</Badge>
                    : <Badge tone="critical">Refunded</Badge>}
          </IndexTable.Cell>


          {selectedAPI === 1 && (
          <IndexTable.Cell>
            {order.pickup_status === null?'Not Pushed':order.pickup_status}
          </IndexTable.Cell>
          )}
          {selectedAPI === 1 && (
          <IndexTable.Cell>
  {companyDetails.length > 1 ? (
    <Select
      options={CompanyOptions}
      onChange={(value) => handleCompanyChange(order.id, value)}
      value={selectedCompanyIds[order.id] || ''}
    />
  ) : (
    companyDetails.length === 1 && <span>{companyDetails[0].company_name}</span>
  )}
</IndexTable.Cell>
 )}

            <IndexTable.Cell>
            {loadingRows[order.id] ? (
        <Spinner accessibilityLabel="Spinner example" size="small" />
      ) : (
        <Button
          dataPrimaryLink
          variant="tertiary"
          onClick={(e) => {
            e.stopPropagation();
            selectedAPI === 1
          ? handleViewDetails(order.id)
          : handleSkyExPush(order.id);
          }}
        >
          Push To SkyEx
        </Button>
        
      )}
     
          </IndexTable.Cell>

          {selectedAPI === 1 && (
          <IndexTable.Cell>
            {reloadLoader[order.id] ? (
        <Spinner accessibilityLabel="Spinner example" size="small" />
      ) : (
        <Button
          dataPrimaryLink
          variant="tertiary"
          onClick={(e) => {
            e.stopPropagation();
            handleReload(order.id);
          }}
        >
          Reload Data
        </Button>
      )}
          </IndexTable.Cell>

    )}
        </IndexTable.Row>
      );
    });

    const rowMarkupRemove = paginatedOrderRemove?.map((order, index) => {
   
      const uniqueVendors = new Set();
  
      return (
      
        <IndexTable.Row
          id={order.id}
          key={order.id}
         selected={selectedResourcesRemove.includes(order.id)}
          position={index}
          onClick={(e) => {
            e.stopPropagation(); 
          }}
        >
          <IndexTable.Cell>
            {((currentPageRemove - 1) * itemsPerPage) + (index + 1)}
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {new Date(order.order_data.created_at).toLocaleDateString(('en-GB'), {
                year: 'numeric',
                month: '2-digit',
                day: '2-digit',
              })}
            </Text>
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {order.order_data.order_number}
            </Text>
          </IndexTable.Cell>
  
          <IndexTable.Cell>
  {order.order_data.customer?.first_name} {order.order_data.customer?.last_name}
</IndexTable.Cell>
  
           
          <IndexTable.Cell>{order.order_data.currency} {order.order_data.current_total_price}</IndexTable.Cell>
          <IndexTable.Cell>
            {order.order_data.financial_status === "pending"
              ? <Badge progress="incomplete" tone="attention">Pending</Badge>
              : order.order_data.financial_status === "paid"
                ? <Badge tone="success" progress="complete">Paid</Badge>
                : order.order_data.financial_status === "partially_paid"
                  ? <Badge progress="partiallyComplete" tone="warning">Partially paid</Badge>
                  : order.order_data.financial_status === "partially_refunded"
                    ? <Badge progress="partiallyComplete" tone="critical">Partially refunded</Badge>
                    : <Badge tone="critical">Refunded</Badge>}
          </IndexTable.Cell>

          {selectedAPI === 1 && (
          <IndexTable.Cell>
            {order.pickup_status === null?'Not Pushed':order.pickup_status}
          </IndexTable.Cell>
          )}

{selectedAPI === 1 && (
          <IndexTable.Cell>
  {companyDetails.length > 1 ? (
    <Select
      options={CompanyOptions}
      onChange={(value) => handleCompanyChange(order.id, value)}
      value={selectedCompanyIds[order.id] || ''}
    />
  ) : (
    companyDetails.length === 1 && <span>{companyDetails[0].company_name}</span>
  )}
</IndexTable.Cell>
)}
            <IndexTable.Cell>
            {loadingRows[order.id] ? (
        <Spinner accessibilityLabel="Spinner example" size="small" />
      ) : (

        <Button
          dataPrimaryLink
          variant="tertiary"
          onClick={(e) => {
            e.stopPropagation();
            selectedAPI === 1
          ? handleViewDetails(order.id)
          : handleSkyExPush(order.id);
          }}
        >
          Push To SkyEx
        </Button>
       
      )}
          </IndexTable.Cell>

          {selectedAPI === 1 && (
          <IndexTable.Cell>
        {reloadLoaderRemove[order.id] ? (
        <Spinner accessibilityLabel="Spinner example" size="small" />
      ) : (
        <Button
          dataPrimaryLink
          variant="tertiary"
          onClick={(e) => {
            e.stopPropagation();
            handleReloadRemove(order.id);
          }}
        >
          Reload Data
        </Button>
      )}
          </IndexTable.Cell>
          )}
        </IndexTable.Row>
      );
    });
    const pushedRowMarkup = pushedPaginatedOrders?.map((order, index) => {
    
      return (
      
        <IndexTable.Row
          id={order.id}
          key={order.id}
         selected={selectedResourcesPush.includes(order.id)}
          position={index}
          onClick={(e) => {
            e.stopPropagation(); 
          }}
        >
          <IndexTable.Cell>
            {((pushedCurrentPage - 1) * itemsPerPage) + (index + 1)}
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {new Date(order.order_data.created_at).toLocaleDateString(('en-GB'), {
                year: 'numeric',
                month: '2-digit',
                day: '2-digit',
              })}
            </Text>
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {order.order_data.order_number}
            </Text>
          </IndexTable.Cell>
  
          <IndexTable.Cell>
  {order.order_data.customer?.first_name} {order.order_data.customer?.last_name}
</IndexTable.Cell>
  
           
          <IndexTable.Cell>{order.order_data.currency} {order.order_data.current_total_price}</IndexTable.Cell>
          <IndexTable.Cell>
            {order.order_data.financial_status === "pending"
              ? <Badge progress="incomplete" tone="attention">Pending</Badge>
              : order.order_data.financial_status === "paid"
                ? <Badge tone="success" progress="complete">Paid</Badge>
                : order.order_data.financial_status === "partially_paid"
                  ? <Badge progress="partiallyComplete" tone="warning">Partially paid</Badge>
                  : order.order_data.financial_status === "partially_refunded"
                    ? <Badge progress="partiallyComplete" tone="critical">Partially refunded</Badge>
                    : <Badge tone="critical">Refunded</Badge>}
          </IndexTable.Cell>
          <IndexTable.Cell>
            {order.awb_no}
          </IndexTable.Cell>
          <IndexTable.Cell>
          {order.pickup_status}
          </IndexTable.Cell>
          <IndexTable.Cell>
  {loadingRowsDownload[order.id] ? (
    <Spinner accessibilityLabel="Downloading..." size="small" />
  ) : (
   
      <Button
      dataPrimaryLink
      variant="tertiary"
      onClick={(e) => {
        e.stopPropagation();
        selectedAPI === 1
      ? handleDownload(order.id)
      : handleDownloadSkyEx(order.id);
      }}
      >
      Download
      </Button>


  )}
</IndexTable.Cell>
        </IndexTable.Row>
      );
    });
  
    const returnApproveSubmit = (returnId, orderNumber,lineItemId,orderId) => {
      const payload = {
        returnId: returnId,
        orderNumber: orderNumber,
        lineItemId:lineItemId,
        orderId:orderId
      };
    
      console.log(payload);
      fetch('/api/returnApproveSubmit', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify(payload),
      })
      .then(response => {
        if (!response.ok) {
          throw new Error('Network response was not ok');
        }
        return response.json(); // Parse the JSON response
      })
      .then(data => {
        console.log(data);
     
        setReturnStatuses(prev => ({...prev, [returnId]: 'Approved'}));
        setDisabledCheckboxes((prev) => ({ ...prev, [returnId]: true }));
        fetchOrderDetails();
        fetchOrderSkyexDetails();
        
        setToastProps({
          content: "Order return request approved",
        });
       
        if (data.error) {
          console.log(data.error);
          setToastProps({
            content: data.error,
            error: true,
          });
        }
      })
      .catch(error => { 
        console.error('Error:', error);
        setToastProps({
          content: error.message, 
          error: true,
        });
      });
    };

    const returnDeclineSubmit = (returnId, orderNumber,declineReason,lineItemId,orderId) => {
      const payload = {
        returnId: returnId,
        orderNumber: orderNumber,
        declineReason: declineReason,
        lineItemId:lineItemId,
        orderId:orderId
      };
    
      console.log(payload);
      fetch('/api/returnDeclineSubmit', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify(payload),
      })
      .then(response => {
        if (!response.ok) {
          throw new Error('Network response was not ok');
        }
        return response.json(); // Parse the JSON response
      })
      .then(data => {

        setReturnStatuses(prev => ({...prev, [returnId]: 'Declined'}));
        setDisabledCheckboxes((prev) => ({ ...prev, [returnId]: true }));
        fetchOrderDetails();
        fetchOrderSkyexDetails();

        setToastProps({
          content: "Order return request declined",
        });
       
        if (data.error) {
          console.log(data.error);
          setToastProps({
            content: data.error,
            error: true,
          });
        }
      })
      .catch(error => { 
        console.error('Error:', error);
        setToastProps({
          content: error.message, 
          error: true,
        });
      });
    };


    const declineModal = (
      <Modal
        open={isDeclineModalOpen}
        onClose={() => {
          setIsDeclineModalOpen(false);
          setDeclineReason('');
          setDeclineReasonError('');
        }}
        title="Decline Return Request"
        primaryAction={{
          content: 'Confirm',
          onAction: () => {
            if (declineReason === '') {
              setDeclineReasonError('Please select a reason for declining');
            } else {
              returnDeclineSubmit(currentReturnId, currentOrderNumber, declineReason,currentLineItemId,currentOrderId);
              setIsDeclineModalOpen(false);
              setDeclineReason('');
              setDeclineReasonError('');
            }
          },
        }}
        secondaryActions={[
          {
            content: 'Cancel',
            onAction: () => {
              setIsDeclineModalOpen(false);
              setDeclineReason('');
              setDeclineReasonError('');
            },
          },
        ]}
      >
        <Modal.Section>
          <Select
            label="Reason for decline"
            options={[
              {label: 'Select reason', value: ''},
              {label: 'Return window closed', value: 'RETURN_PERIOD_ENDED'},
              {label: 'Final sale', value: 'FINAL_SALE'},
              {label: 'Other', value: 'OTHER'},
            ]}
            onChange={(value) => {
              setDeclineReason(value);
              setDeclineReasonError('');
            }}
            value={declineReason}
            error={declineReasonError}
          />
        </Modal.Section>
      </Modal>
    );
    
    const [allReturnLineItemsSelected, setAllReturnLineItemsSelected] = useState({});
    const [selectedReturnLineItems, setSelectedReturnLineItems] = useState([]);

    const rowMarkupReturn = paginatedOrderReturn?.map((order, index) => {
    
      const handleItemSelect = (itemId, orderId) => {
        setSelectedReturnLineItems(prevSelected => {
          const newSelected = prevSelected.includes(itemId)
            ? prevSelected.filter(id => id !== itemId)
            : [...prevSelected, itemId];
          
          // Check if all non-disabled items for this order are selected
          const itemsInOrder = JSON.parse(order.returnLineItem_details);
          const allNonDisabledItems = itemsInOrder
            .map((item, index) => {
              const returnDetails = JSON.parse(order.return_details);
              const fullReturnId = returnDetails[index]?.id || '';
              const returnId = fullReturnId.split('/').pop() || 'N/A';
              return !disabledCheckboxes[returnId];
            })
            .filter(Boolean);
      
          const allSelectedCount = newSelected.filter(id => {
            const [orderNumber, returnId] = id.split('-');
            return orderNumber === order.order_number && !disabledCheckboxes[returnId];
          }).length;
      
          setAllReturnLineItemsSelected(prev => ({
            ...prev,
            [orderId]: allSelectedCount === allNonDisabledItems.length
          }));
          
          return newSelected;
        });
      };

      const handleSelectAllCheckbox = (orderId) => () => {
        const newAllSelected = !allReturnLineItemsSelected[orderId];
        setAllReturnLineItemsSelected(prev => ({
          ...prev,
          [orderId]: newAllSelected
        }));
        
        const itemsInOrder = JSON.parse(order.returnLineItem_details).map((item, index) => {
          const lineItemId = item[0].lineItemId;
          const returnDetails = JSON.parse(order.return_details);
          const fullReturnId = returnDetails[index]?.id || '';
          const returnId = fullReturnId.split('/').pop() || 'N/A';
          const itemId = `${order.order_number}-${returnId}-${lineItemId}-${orderId}`;
          return { id: itemId, disabled: disabledCheckboxes[returnId] };
        });
      
        if (newAllSelected) {
          setSelectedReturnLineItems(prevSelected => 
            [...new Set([...prevSelected, ...itemsInOrder.filter(item => !item.disabled).map(item => item.id)])]
          );
        } else {
          setSelectedReturnLineItems(prevSelected => 
            prevSelected.filter(item => !itemsInOrder.find(orderItem => orderItem.id === item))
          );
        }
      };
  
     
      const isAnyItemSelected = (orderId) => {
        return selectedReturnLineItems.some(itemId => itemId.includes(orderId));
      };

      const shouldEnablePushToSkyex = (orderId) => {
     
        try {
          const order = paginatedOrderReturn.find(o => o.id === orderId);
          if (!order || !order.approve_decline_details) {
            return false;
          }
      
          const returnDetails = JSON.parse(order.approve_decline_details);
          
          let awbItems = 0;
          if (order.awb_number_details) {
            const awbNumberDetails = JSON.parse(order.awb_number_details);
            awbItems = awbNumberDetails.length;
          }
      
          const openReturns = returnDetails.filter(item => item.status === 'OPEN').length;
          //console.log(returnDetails);
          return openReturns !== awbItems;
        } catch (error) {
          console.error("Error in shouldEnablePushToSkyex:", error);
          return false;
        }
      };
      
      
      return (
        <>
      
        <IndexTable.Row  
        onClick={(e) => {
              e.stopPropagation(); 
            }}
          id={order.id}
          key={order.id}
         selected={selectedResourcesReturn.includes(order.id)}
          position={index}
        >
          <IndexTable.Cell>
            <div class="arrow_button">
            <Button 
            plain 
            icon={expandedRowId === order.id ? ChevronUpMinor : ChevronDownMinor} 
            onClick={(e) => {
              e.stopPropagation(); 
              toggleRow(order.id);
            }}
            />
            </div>
          </IndexTable.Cell>

          <IndexTable.Cell>
            {((currentPageReturn - 1) * itemsPerPage) + (index + 1)}
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {new Date(order.created_at).toLocaleDateString(('en-GB'), {
                year: 'numeric',
                month: '2-digit',
                day: '2-digit',
              })}
            </Text>
          </IndexTable.Cell>
          <IndexTable.Cell>
            <Text variant="bodyMd" fontWeight="bold" as="span">
              {order.order_number}
            </Text>
          </IndexTable.Cell>
  
          <IndexTable.Cell>
          {order.customer_first_name}
          </IndexTable.Cell>
  
          <IndexTable.Cell>{order.totalReturnAmount}</IndexTable.Cell>
          <IndexTable.Cell>
           
          {(() => {
          let overallStatus = 'Approved';
          let hasApproved = false;
          let hasDeclined = false;
          let hasPending = false;
          
          JSON.parse(order.returnLineItem_details).forEach((item, index) => {
            const returnDetails = JSON.parse(order.return_details);
            const fullReturnId = returnDetails[index]?.id || '';
            const returnId = fullReturnId.split('/').pop() || 'N/A';
            const status = returnStatuses[returnId];

            if (status === 'Approved') {
              hasApproved = true;
            } else if (status === 'Declined') {
              hasDeclined = true;
            } else {
              hasPending = true;
            }
          });

          if (hasPending) {
            overallStatus = 'Pending';
          } else if (hasApproved && hasDeclined) {
            overallStatus = 'Partially approved';
          } else if (hasDeclined) {
            overallStatus = 'Declined';
          }

          return <div>{overallStatus}</div>;
        })()}


          </IndexTable.Cell>
            <IndexTable.Cell>
            {loadingRows[order.id] ? (
        <Spinner accessibilityLabel="Spinner example" size="small" />
      ) : (
        <div class="returnPushButton">
        <Button
          dataPrimaryLink
          variant="tertiary"
          onClick={() => handleReturnPushToSkyex(order.id)}
          disabled={!shouldEnablePushToSkyex(order.id)}
        >
          Push To SkyEx
        </Button>
        </div>
      )}
          </IndexTable.Cell>
        </IndexTable.Row>

       {expandedRowId === order.id && (
        <tr key={`${order.id}-details-row`}>
          <td colSpan="9">
            <div style={{ padding: '16px', backgroundColor: '#f4f6f8', borderTop: '1px solid #dfe3e8' }}>
            
              <IndexTable
                resourceName={resourceNames}
                itemCount={1}
                headings={[										
                  { title: 'Order' },
                  { title: 'Customer' },
                  { title: 'Total' },							
                ]}
                selectable={false}
              >
                <IndexTable.Row id={`${order.id}-details`} position={0}>
                  <IndexTable.Cell>
                    <Text variant="bodyMd" as="span">
                      {order.order_number}
                    </Text>
                  </IndexTable.Cell>
                  <IndexTable.Cell>
                    {order.customer_first_name}
                  </IndexTable.Cell>
                  <IndexTable.Cell>
                    {order.totalReturnAmount}
                  </IndexTable.Cell>
                </IndexTable.Row>
              </IndexTable>
             
              <div className='returnProductsIndexTable'>
                <IndexTable
                  id={order.id}
                  resourceName={resourceNames}
                  itemCount={JSON.parse(order.returnLineItem_details).length}
                  position={index}
                  headings={[
                    { 
                      title: (
                        <Checkbox
                          label="Select all"
                          labelHidden
                          checked={allReturnLineItemsSelected[order.id] || false}
                          onChange={handleSelectAllCheckbox(order.id)}
                        />
                      ),
                      
                    },
                    { title: 'Product Name' },
                    { title: 'Quantity' },
                    { title: 'Amount' },
                    { title: 'Return Reason' },
                    { title: 'Actions' },
                    { title: 'AWB & Label' },
                    { title: '' },
                    { title: '' },
                  ]}
                  selectable={false}
                  //promotedBulkActions={promotedBulkActionsReturnProducts}
                  promotedBulkActions={isAnyItemSelected(order.id) ? promotedBulkActionsReturnProducts : []}
                  selectedItemsCount={
                    allReturnLineItemsSelected ? 'All' : selectedReturnLineItems.length
                  }
                  onSelectionChange={setSelectedReturnLineItems}
                  lastColumnSticky
                >
                  {JSON.parse(order.returnLineItem_details).map((item, index) => {

                     if (!Array.isArray(item) || item.length === 0) {
                      console.error("Invalid item format:", item);
                      return null; 
                  }
              
                  const itemDetails = item[0]; 
              
                  if (typeof itemDetails !== 'object' || itemDetails === null) {
                      console.error("Invalid itemDetails format:", itemDetails);
                      return null; 
                  }
             
                    const quantity = itemDetails.quantity || 0; 
                    // const quantity = item[0].quantity;
                    const price = parseFloat(item[0].productPrice);
                    const lineItemId = item[0].lineItemId;
                    const amount = (quantity * price).toFixed(2);
                    const returnDetails = JSON.parse(order.return_details);
                    const fullReturnId = returnDetails[index]?.id || '';
                    const returnId = fullReturnId.split('/').pop() || 'N/A';
                    const orderNumber = order.order_number;
                    const orderId = order.id;
                    //const itemId = `${order.id}-${index}`;
                    const itemId = `${order.order_number}-${returnId}-${lineItemId}-${orderId}`;
                   
                    //const awbNumberDetails = JSON.parse(order.awb_number_details);
                    //const awbNumber = awbNumberDetails.find(detail => detail.line_item_id.toString() === lineItemId.toString())?.awb_number || 'N/A';
                    
                    let awbNumber = 'N/A';

                    if (order.awb_number_details) {
                        try {
                            const awbNumberDetails = JSON.parse(order.awb_number_details);
                            if (Array.isArray(awbNumberDetails)) {
                                const foundDetail = awbNumberDetails.find(detail => 
                                    detail.line_item_id.toString() === lineItemId.toString()
                                );
                                if (foundDetail) {
                                    awbNumber = foundDetail.awb_number;
                                }
                            }
                        } catch (error) {
                            console.error("Error parsing awb_number_details:", error);
                        }
                    }

                    return (
                      <IndexTable.Row 
                        id={itemId} 
                        key={itemId} 
                       // selected={selectedReturnLineItems.includes(itemId)}
                       // selected={selectedReturnLineItems.includes(itemId) && !disabledCheckboxes[returnId]}
                        selected={!disabledCheckboxes[returnId] && (allReturnLineItemsSelected || selectedReturnLineItems.includes(itemId))}
                        position={index}
                        disabled={disabledCheckboxes[returnId]} 
                        onClick={(e) => {
                          e.stopPropagation();
                        }}
                      >
                        <IndexTable.Cell>
                      <Checkbox
                        label={`Select item ${index + 1}`}
                        labelHidden
                        checked={!disabledCheckboxes[returnId] && (allReturnLineItemsSelected[order.id] || selectedReturnLineItems.includes(itemId))}
                        disabled={disabledCheckboxes[returnId]}
                        onChange={() => handleItemSelect(itemId, order.id)}
                      />
                    </IndexTable.Cell>
                        <IndexTable.Cell>
                          <Text variant="bodyMd" as="span">
                            {item[0].productName}
                          </Text>
                        </IndexTable.Cell>
                        <IndexTable.Cell>{quantity}</IndexTable.Cell>
                        <IndexTable.Cell>{amount}</IndexTable.Cell>
                        <IndexTable.Cell>
                          <Text variant="bodyMd" as="p">
                            {item[0].returnReason}
                          </Text>
                          {item[0].customerNote && (
                            <Text variant="bodyMd" as="p" color="subdued">
                              {item[0].customerNote}
                            </Text>
                          )}
                        </IndexTable.Cell>
                        <IndexTable.Cell>
                          {returnStatuses[returnId] ? (
                            returnStatuses[returnId] === 'Approved' ? (
                              <span style={{ 
                                backgroundColor: '#AEE9D1', 
                                color: '#108043', 
                                padding: '2px 8px', 
                                borderRadius: '3px', 
                                fontWeight: '500' 
                              }}>
                                Approved
                              </span>
                            ) : (
                              <span style={{ 
                                backgroundColor: '#FED3D1', 
                                color: '#D82C0D', 
                                padding: '2px 8px', 
                                borderRadius: '3px', 
                                fontWeight: '500' 
                              }}>
                                Declined
                              </span>
                            )
                          ) : (
                            <>
                              <Button onClick={() => returnApproveSubmit(returnId, orderNumber,lineItemId,orderId)}>Approve</Button>
                              <Button onClick={() => {
                                setIsDeclineModalOpen(true);
                                setCurrentReturnId(returnId);
                                setCurrentOrderNumber(orderNumber);
                                setCurrentLineItemId(lineItemId);
                                setCurrentOrderId(orderId);
                              }}>
                                Decline
                              </Button>
                            </>
                          )}
                        </IndexTable.Cell>
                        <IndexTable.Cell>
                          <div>
                            {awbNumber}
                          </div>
                        </IndexTable.Cell>
                        <IndexTable.Cell>
                        <div>
                            {awbNumber !== 'N/A' && (
                                loadingRowsDownload[lineItemId] ? (
                                  <Spinner accessibilityLabel="Downloading..." size="small" />
                                ) : (
                                  <Button
                                    dataPrimaryLink
                                    variant="tertiary"
                                    onClick={() => handleReturnDownload(order.id,lineItemId)}
                                  >
                                    Download
                                  </Button>
                                )
                              )}
                          </div>
                        </IndexTable.Cell>
                        <IndexTable.Cell>
                          <div style={{display:'none'}}>
                            {returnId}
                          </div>
                        </IndexTable.Cell>
                      </IndexTable.Row>
                    );
                  })}
                </IndexTable>
              </div>
            </div>
          </td>
        </tr> 
      )}


        </>
      );
    });

    return (
        <Frame>
    <Page fullWidth>
    {toastMarkup}
      <TitleBar title={t("HomePage.title")} primaryAction={null} />
      <LegacyCard sectioned>
      <div style={{ display: 'flex', justifyContent: 'center',marginBottom:'30px' }}>
                <Image
                    alt="Sample Image"
                    source={skyexlogo}
                   width={200}
                />
                </div>
            </LegacyCard>
      <Layout>
        <Layout.Section>
          <LegacyCard>
            <div class="TabsDiv">
            <Tabs tabs={tabs} selected={selectedTab} onSelect={handleTabChange} fitted>
              {tabs.map((tab, index) => (
                index === selectedTab && (
                  <LegacyCard.Section key={tab.id}>
                    {tab.renderContent()}
                  </LegacyCard.Section>
                )
              ))}
            </Tabs>
            </div>
          </LegacyCard>
        </Layout.Section>
        <Modal
        open={active}
        onClose={handleModalChange}
        title="Change Location"
        primaryAction={{
          content: 'Save',
          onAction: handleSave,
        }}
        secondaryActions={[
          {
            content: 'Cancel',
            onAction: handleModalChange,
          },
        ]}
      >
        <Modal.Section>
          <Select
            label="Select a location"
            options={CompanyOptions}
            onChange={handleSelectChange}
            value={selectedModal}
          />
        </Modal.Section>
      </Modal>

      <Modal
        open={activeRemove}
        onClose={handleModalChangeRemove}
        title="Change Location"
        primaryAction={{
          content: 'Save',
          onAction: handleSaveRemove,
        }}
        secondaryActions={[
          {
            content: 'Cancel',
            onAction: handleModalChangeRemove,
          },
        ]}
      >
        <Modal.Section>
          <Select
            label="Select a location"
            options={CompanyOptions}
            onChange={handleSelectChangeRemove}
            value={selectedModalRemove}
          />
        </Modal.Section>
      </Modal>
      </Layout>
      
    </Page>
    {declineModal}
    </Frame>
      );

      function disambiguateLabel(key, value) {
        switch (key) {
          case 'paymentStatus':
            return (value).map((val) => `${val}`).join(', ');
          case 'accountStatus':
            return (value).map((val) => `${val}`).join(', ');
          case 'dateStatus':
            if (value.includes('Custom Date')) {
              const formattedStartDate = selectedDates.start.toLocaleDateString(('en-GB'));
              const formattedEndDate = selectedDates.end.toLocaleDateString();
              return `Date: ${formattedStartDate} - ${formattedEndDate}`;
            } else {
              return value.map(val => `Date: ${val}`).join(', ');
            }
          default:
            return value;
        }
      }
    
      function isEmpty(value) {
        if (Array.isArray(value)) {
          return value.length === 0;
        } else {
          return value === '' || value == null;
        }
      }
}