import React, { useState, useCallback,useEffect  } from 'react';
import {
  Page,
  Layout,
  LegacyCard,
  Tabs,
  Form,
  FormLayout,
  TextField,
  Button,
  Toast,
  Text,
  Frame ,EmptyState, Link, Image , Select, Checkbox, LegacyStack, RadioButton
} from "@shopify/polaris";
import { TitleBar } from "@shopify/app-bridge-react";
import { useTranslation } from "react-i18next";
import { useAppQuery, useAuthenticatedFetch } from "../hooks";
import './style.css';
import { domesticImage } from "../assets";
import { importImage } from "../assets";
import { exportImage } from "../assets";
import { skyexlogo } from "../assets";
import Orders from './Orders';
import Settings from './Settings';
import { useNavigate , useLocation } from 'react-router-dom';
import { update } from '@shopify/app-bridge/actions/MarketingExternalActivityTopBar';



export default function SettingsOld() {

  const navigate = useNavigate();
const location = useLocation();
  const fetch = useAuthenticatedFetch();
  const emptyToastProps = { content: null };
	const [toastProps, setToastProps] = useState(emptyToastProps); 
  const toastMarkup = toastProps.content && (
		<Toast {...toastProps} onDismiss={() => setToastProps(emptyToastProps)} />
	);


  // State for SkyEx details and form
  const [skyexApikey, setskyexApikey] = useState('');
  const [weight, setWeight] = useState('');
  const [account, setAccount] = useState('');
  const [shipperType, setshipperType] = useState('');
  
  // State for button label: "Submit" or "Update"
  const [isUpdating, setIsUpdating] = useState(false);

  // Error States
  const [apiKeyError, setApiKeyError] = useState('');
  const [weightError, setWeightError] = useState('');
  const [accountError, setAccountError] = useState('');
  const [shipperTypeError, setShipperTypeError] = useState('');


 

  // Fetch details from backend to check if the model is empty or not
  useEffect(() => {
    const fetchSkyexDetails = async () => {
      try {
        const response = await fetch('/api/getDetails');
        const data = await response.json();
        if (data && data.api_key) {
          // Data exists, so this is an update action
          setskyexApikey(data.api_key);
          setWeight(data.weight);
          setAccount(data.account);
          console.log(data.account);
          setshipperType(data.shipper_type);
          setIsUpdating(true); // Change button to "Update" and enable Delete
        } else {
          setIsUpdating(false); // Change button to "Submit" and disable Delete
        }
      } catch (error) {
        console.error('Error fetching SkyEx details:', error);
      }
    };

    fetchSkyexDetails();
  }, []);

  
  // Text Field Handlers

  
  const handleAccountNumberChange = useCallback(
    (newValue) => {
      setAccount(newValue);
      setAccountError(''); // Reset error on change
    },
    []
  );

  const handleApiKeyChange = useCallback(
    (newValue) => {
      setskyexApikey(newValue);
      setApiKeyError(''); // Reset error on change
    },
    []
  );

  const handleWeightChange = useCallback(
    (newValue) => {
      setWeight(newValue);
      setWeightError(''); // Reset error on change
    },
    []
  );

  // Radio Button Handler
  const handleRadio = useCallback(
    (_, newValue) => {
      setshipperType(newValue);
      setShipperTypeError(''); // Reset error on change
    },
    []
  );

  // Handle form submission (both submit and update)
  const handleViewDetails = async () => {
    // Reset errors
    setApiKeyError('');
    setWeightError('');
    setShipperTypeError('');

    let valid = true;

    // Validation
    if (!skyexApikey) {
      setApiKeyError('API Key is required.');
      valid = false;
    }

    if (!shipperType) {
      setShipperTypeError('Please select a shipment type.');
      valid = false;
    }

    if (!account) {
      setAccountError('Please enter account number.');
      valid = false;
    }

    if (!weight || isNaN(weight) || parseFloat(weight) <= 0) {
      setWeightError('Please enter a valid weight greater than 0.');
      valid = false;
    }

    if (!valid) return; // Stop execution if validation fails

    // Proceed with the API call if all validation passes
    try {
      const response = await fetch('/api/saveskyexdetial', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          skyexApikey: skyexApikey,
          weight: weight,
          account: account,
          shipperType: shipperType,
        }),
      });

      if (response.ok) {
        setToastProps({
          content: "Settings saved to SkyEx successfully",
        });
        setTimeout(() => {
          window.location.reload();
        }, 1000);
      } else if (response.status === 401) {
        const data = await response.json();
        setToastProps({
          content: data.error || "Invalid API Key. Please provide a valid one.",
          error: true,
        });

        // Revert to old data if the API key is invalid
        if (data.oldData) {
          setskyexApikey(data.oldData.api_key);
          setWeight(data.oldData.weight);
          setAccount(data.oldData.account);
          setshipperType(data.oldData.shipper_type);
        }
      } else {
        const data = await response.json();
        setToastProps({
          content: data.error || "An error occurred",
          error: true,
        });
      }
    } catch (error) {
      console.error('Error fetching data:', error);
      setToastProps({
        content: "Failed to push to SkyEx",
        error: true,
      });
    }
  };

  // Handle delete details
  const deleteDetails = async () => {
    try {
      const response = await fetch('/api/deleteDetails', {
        method: 'DELETE',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          skyexApikey: skyexApikey,
          weight: weight,
          account: account,
          shipperType: shipperType,
        }),
      });

      if (response.ok) {
        setToastProps({
          content: "Deleted from SkyEx successfully",
        });
        setTimeout(() => {
          window.location.reload();
        }, 1000);
      } else {
        const data = await response.json();
        setToastProps({
          content: data.error,
          error: true,
        });
      }
    } catch (error) {
      console.error('Error fetching data:', error);
    }
  };

  // Handle Clear Button
  const handleClear = () => {
    // Clear all input fields
    setskyexApikey('');
    setWeight('');
    setAccount('');
    setshipperType('');

    // Reset error states
    setApiKeyError('');
    setWeightError('');
    setAccountError('');
    setShipperTypeError('');
  };


  
  const handleChangeApiClick = () => {
       
    
    const payload = {
      buttonChangeApiStatus: null,
    };
    const url = '/api/buttonChangeApiStatus';
    fetch(url, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify(payload),
    })
    .then(response => {
      
      if (!response.ok) {
        throw new Error('Network response was not ok');
      
      }
     
      if (location.pathname === '/EntryPage') {
        navigate('/'); // If on EntryPage, redirect to index page
      } else {
        navigate('/EntryPage'); // If on index, redirect to EntryPage
      }
      // if (onChangeApi) {
      //   onChangeApi(); // This will reset the API selection in index.jsx
      // }
    })
    .catch(error =>{ console.error('Error:', error);
      // setShowApiToast(true);
      // setIsLoadingAccount(false);
  });  

  // if (location.pathname === '/EntryPage') {
  //   navigate('/'); // If on EntryPage, redirect to index page
  // } else {
  //   navigate('/EntryPage'); // If on index, redirect to EntryPage
  // }

 
  
 
};

return (
    <Frame>
      <Page narrowWidth>
        {toastMarkup}
        <LegacyCard sectioned>
          <div style={{ display: 'flex', justifyContent: 'center' }}>
            <Image alt="Sample Image" source={skyexlogo} width={200} height={200} />
          </div>
        </LegacyCard>
        <LegacyCard sectioned>
        {/* <div className='changeAPIsettingsButton'><Button onClick={handleChangeApiClick}>Change API</Button></div> */}

          {/* API Key */}

          <div>
            <TextField
              label={<b> Account Number</b>}
              value={account}
              onChange={handleAccountNumberChange}
              autoComplete="off"
              error={accountError ? accountError : null} // Show error if exists
            />
          </div><br></br><br></br>

          <div>
            <TextField
              label={<b>API Key</b>}
              value={skyexApikey}
              onChange={handleApiKeyChange}
              multiline={4}
              autoComplete="off"
              error={apiKeyError ? apiKeyError : null} // Show error if exists
            /><br></br><br></br>
          </div>

          {/* Select the Shipment Type */}
          <div>
            <Text> <b>Select the Shipment Type:</b> </Text>
            <LegacyStack horizontal>
              <RadioButton
                label="Document"
                checked={shipperType === 'document'}
                id="document"
                name="accounts"
                onChange={handleRadio}
              />
              <RadioButton
                label="Non Document"
                id="NonDocument"
                name="accounts"
                checked={shipperType === 'NonDocument'}
                onChange={handleRadio}
              />
            </LegacyStack>
            {shipperTypeError && (
              <p style={{ color: 'red' }}>{shipperTypeError}</p>
            )}
            <br /><br />
          </div>

          {/* Default product weight in kg */}
          <div>
            <TextField
              label={<b> Default product weight in kg:</b>}
              value={weight}
              onChange={handleWeightChange}
              autoComplete="off"
              error={weightError ? weightError : null} // Show error if exists
            />
          </div><br></br><br></br>
          <div>
            <div style={{ display: 'flex', gap: '40px' }}>
              <div className="custom-button">
                <Button className="custom-button" primary onClick={handleViewDetails}>
                  {isUpdating ? "Update" : "Submit"}
                </Button>
              </div>
              {/* <div className="custom-delete-button">
                <Button className="custom-delete-button"
                  default
                  disabled={!isUpdating} // Disable delete if the data is not for update
                  onClick={deleteDetails}
                >
                  Delete
                </Button>
              </div> */}
              <div>
                <Button
                  default
                  onClick={handleClear}
                >
                  Clear
                </Button>
              </div>
            </div>
          </div>
        </LegacyCard>
      </Page>
    </Frame>
  );
}
