<?php

use App\Models\Order;
use App\Models\AccessDetail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\OrderController;
use App\Http\Controllers\ReturnController;
use App\Http\Controllers\DetailsController;
use App\Http\Controllers\WebhookController;
use App\Http\Controllers\SkyExOrderController;
use App\Http\Controllers\SkyexDetailsController;
use Illuminate\Support\Facades\Response;
/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::get('/', function () {
    return "Hello API";
})->middleware('shopify.auth');


Route::post('/buttonChangeApiStatus',[DetailsController::class,'buttonChangeApiStatus'])->middleware('shopify.auth');
Route::post('/addApiStatus',[DetailsController::class,'addApiStatus'])->middleware('shopify.auth');
Route::post('/changeApiStatus',[DetailsController::class,'changeApiStatus'])->middleware('shopify.auth');
Route::post('/skyExGetAWBNumber',[SkyExOrderController::class,'skyExGetAWBNumber'])->middleware('shopify.auth');
Route::get('/getSkyexOrders',[SkyExOrderController::class,'getSkyexOrders'])->middleware('shopify.auth');
Route::post('/getDownloadLabelSkyEx',[SkyExOrderController::class,'getDownloadLabelSkyEx'])->middleware('shopify.auth');
Route::post('/getRemoveAllSkyEx',[SkyExOrderController::class,'getRemoveAllSkyEx'])->middleware('shopify.auth');
Route::post('/getDownloadLabelAllSkyEx',[SkyExOrderController::class,'getDownloadLabelAllSkyEx'])->middleware('shopify.auth');


Route::get('/download-pdf/{filename}', function ($filename) {
    $path = storage_path('app/public/pdf/' . $filename);
    
    if (file_exists($path)) {
        return Response::make(file_get_contents($path), 200, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="' . $filename . '"'
        ]);
    } else {
        abort(404, 'PDF file not found');
    }
})->name('download.pdf');


Route::post('/saveskyexdetial',[SkyexDetailsController::class,'saveskyexdetial'])->middleware('shopify.auth');
Route::get('/getDetails',[SkyexDetailsController::class,'getDetails'])->middleware('shopify.auth');
Route::delete('/deleteDetails',[SkyexDetailsController::class,'deleteDetails'])->middleware('shopify.auth');
Route::get('/checkSkyexTableStatus',[SkyexDetailsController::class,'checkSkyexTableStatus'])->middleware('shopify.auth');
Route::get('/checkApiStatusdata',[SkyexDetailsController::class,'checkApiStatusdata'])->middleware('shopify.auth');
Route::get('/checkChangeApiStatusdata',[SkyexDetailsController::class,'checkChangeApiStatusdata'])->middleware('shopify.auth');

Route::get('/checkBackendStatus',[DetailsController::class,'checkBackendStatus'])->middleware('shopify.auth');
Route::get('/getAccessDetails',[DetailsController::class,'getAccessDetails'])->middleware('shopify.auth');
Route::post('/addData',[DetailsController::class,'storeAccessDetails'])->middleware('shopify.auth');
Route::post('/updateDetails', [DetailsController::class, 'updateDetails'])->middleware('shopify.auth');
Route::post('/submitShippingDetails', [DetailsController::class, 'submitShippingDetails'])->middleware('shopify.auth');
Route::post('/updateShippingDetails', [DetailsController::class, 'submitShippingDetails'])->middleware('shopify.auth');

Route::get('/getProductType', [DetailsController::class, 'getProductType'])->middleware('shopify.auth');
Route::get('/getProductService', [DetailsController::class, 'getProductService'])->middleware('shopify.auth');
Route::get('/getPdfSize', [DetailsController::class, 'getPdfSize'])->middleware('shopify.auth');

Route::get('/getCountries', [DetailsController::class, 'getCountries'])->middleware('shopify.auth');
Route::post('/sendCountries', [DetailsController::class, 'sendCountries'])->middleware('shopify.auth');
Route::post('/submitAddressDetails', [DetailsController::class, 'submitAddressDetails'])->middleware('shopify.auth');
Route::post('/saveNewShipperAddress', [DetailsController::class, 'saveNewShipperAddress'])->middleware('shopify.auth');
Route::get('/getCompanies', [DetailsController::class, 'getCompanies'])->middleware('shopify.auth');
Route::delete('/deleteCompany', [DetailsController::class, 'deleteCompany'])->middleware('shopify.auth');

Route::post('/createWebhook', [WebhookController::class, 'createWebhook'])->middleware('shopify.auth');
Route::post('/order-create/{shopId?}', [WebhookController::class, 'orderCreate'])->name('order-create');
Route::post('/order-delete/{shopId?}', [WebhookController::class, 'orderDelete'])->name('order-delete');
Route::post('/order-fulfilled/{shopId?}', [WebhookController::class, 'orderFulfilled'])->name('order-fulfilled');

Route::get('/getOrders',[OrderController::class,'getOrders'])->middleware('shopify.auth');
Route::post('/getAWBNumber',[OrderController::class,'getAWBNumber'])->middleware('shopify.auth');
Route::post('/getDownloadLabel',[OrderController::class,'getDownloadLabel'])->middleware('shopify.auth');
Route::post('/getDownloadLabelAll',[OrderController::class,'getDownloadLabelAll'])->middleware('shopify.auth');
Route::post('/getRemoveAll',[OrderController::class,'getRemoveAll'])->middleware('shopify.auth');

Route::post('/getReloadDataAll',[OrderController::class,'getReloadDataAll'])->middleware('shopify.auth');
Route::post('/getReloadData',[OrderController::class,'getReloadData'])->middleware('shopify.auth');


Route::post('/bulkPushToSkyEx',[OrderController::class,'bulkPushToSkyEx'])->middleware('shopify.auth');


//Route::get('/getServiceListApi',[DetailsController::class,'getServiceListApi']);

Route::post('/statusWebhook', function (Request $request) {
    $payload = json_decode($request->getContent(), true);
   
           if (isset($payload['CustomerCode']) && isset($payload['WayBillNumber']) && isset($payload['Status'])) {
            $customerCode = $payload['CustomerCode'];
            $awb_no = $payload['WayBillNumber'];
            $status = $payload['Status'];
            
            $accessDetail = AccessDetail::where('customer_code', $customerCode)->first();
            
            if ($accessDetail) {
                $order = Order::where('access_id', $accessDetail->id)
                              ->where('awb_no', $awb_no)
                              ->first();
                              
                if ($order) {
                    $order->pickup_status = $status;
                    $order->save();
                    
                    Log::info('Order status updated for AWB number: ' . $awb_no . ' to status: ' . $status);
                } else {
                    Log::info('No order found for AWB number: ' . $awb_no);
                }
            } else {
                Log::info('No Access Detail found for customer code: ' . $customerCode);
            }
        } else {
            Log::warning('Required fields not found in the payload.');
        }
    
        return response()->json(['status' => 'received']);
});


Route::get('/submitReturnDetails/{domain}', [ReturnController::class, 'submitReturnDetails']);
Route::get('/submitReturn/{domain}', [ReturnController::class, 'submitReturn']);
Route::post('/returnApproveSubmit', [ReturnController::class, 'returnApproveSubmit'])->middleware('shopify.auth');
Route::post('/returnDeclineSubmit', [ReturnController::class, 'returnDeclineSubmit'])->middleware('shopify.auth');
Route::post('/getAWBNumberReturn',[ReturnController::class,'getAWBNumberReturn'])->middleware('shopify.auth');
Route::post('/getDownloadLabelReturn',[ReturnController::class,'getDownloadLabelReturn'])->middleware('shopify.auth');
Route::post('/getDownloadLabelAllReturn',[ReturnController::class,'getDownloadLabelAllReturn'])->middleware('shopify.auth');